/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_bank.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM load of FCF Banks
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_bank(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_bank(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    %let obs_count = %fcf_get_observation_count(table=&ds,SASDATASET=Y);
    %if &obs_count < 1 %then %do;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Incoming DS - &ds                                       * 
     * Target Table:    New Banks - work.new_banks                              * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_banks;
    quit;

    proc sql;
        create table work.new_banks as
            select
                SEGMENT_ID,
                BANK_NUMBER,
                BANK_NAME
            from &SOURCEDS
        ;
    quit;

    %let SOURCEDS = work.new_banks;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create new_banks from input table for BNK load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Households - work.new_banks                         * 
     * Target Table:    New Parties - iem.new_parties                           * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.new_banks);

    data _null_;
        call symput('current_dt',put(datetime(),datetime22.));
    run;
    
    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_parties;
    quit;

    data iem.new_parties;
        if 0 then set ecm_db.party_live;
        set &SOURCEDS. (keep= BANK_NUMBER SEGMENT_ID BANK_NAME);
        VALID_FROM_DTTM = "&current_dt"dt;
        PARTY_ID = CATX('-','BNK',BANK_NUMBER);
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        PARTY_TYPE_CD = 'FCFSUB';
        PARTY_CATEGORY_CD = 'BNK';
        INDIVIDUAL_FLG = '0';
        PARTY_FULL_NM = BANK_NAME;
        CREATE_USER_ID = "&ecm_user";
        CREATE_DTTM =  "&current_dt"dt;
        UPDATE_USER_ID = "&ecm_user";
        VERSION_NO = 1;
        DELETE_FLG = "0";
        X_SASFCS_RSK_CLSSF = .;
        drop BANK_NUMBER SEGMENT_ID BANK_NAME;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_PARTIES for BNK load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    %fcf_get_ecm_sequence_ids(in=iem.new_parties,out=iem.new_parties,type=party,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM PARTY primary keys for BNK load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_load_subject(type='bank',ds=iem.new_parties);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
