/* MAINDOC
+----------------------------------------------------------------------------------------------+
| NAME:    ctr_custom_fmt.sas
| PRODUCT: SAS Currency Transaction Reporting
| SYSTEM:  Windows, Unix
| DESC:    Define custom format for CTR
|          This macro should be adjusted to match with the domain values of the source data
|          
+---------------------------------------------------------------------------------------------+
| USAGE:   %ctr_custom_fmt_data;
|          refer to ctr_custom_macro for the usage of these formats     
+---------------------------------------------------------------------------------------------+
MAINDOCEND */

/*---------------------------------------------------------------------------------------------
 * Copyright (c) 2017-2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *--------------------------------------------------------------------------------------------*/

%macro ctr_custom_fmt;



    proc format ;
          /* North American countries in 2-char country codes which require state code */
          value $_na_ctry_2_fmt
          'US', 'CA', 'MX','AS','FM','GU','MH','MP','PR','PW','VI','UM'='Y'
          other='N'
          ;
           /* North American countries in 3-char country codes which require state code */
          value $_na_ctry_3_fmt
          'USA', 'CAN', 'MEX','ASM','FSM','GUM','MHL','MNP','PRI','PLW','VIR','UMI'='Y'
          other='N'
          ;
          
          /* mapping FSC_PARTY_DIM.PARTY_IDENTIFICATION_TYPE_DESC to     X_PERSONAL_ID_TYPE_CD */
          /* A: Unknown                                                                        */
          /* B: Driver license/State ID                                                        */
          /* C: Passport                                                                       */
          /* D: Alien Registration                                                             */

          value $_personal_id_type 
          ' '        ='A'
          'DL', 'STI'='B'          
          'PA'       ='C'
          'GC'       ='D'
          other      ='Z'
          ;
        
          /* mapping FSC_PARTY_DIM.PARTY_IDENTIFICATION_DESC to X_PERSON_INVOLVED.X_PERSONAL_ID_TYPE_OTH_TXT */
          /* when personal_id_type is Z.                                                   */
          value $_personal_other_id_desc 
          'MI'    ='Military License' 
          'EI'   = 'EMPLOYER ID'
          'GI'   ='GOVERNMENT ID'
          'SCI'  ='SCHOOL ID'           
          'UI'   ='UNION ID' 
          'OTHER'='OTHER'
          other = ' ' 
          ;

          /* mapping FSC_PARTY_DIM.PARTY_TAX_ID_TYPE_CODE to X_PERSON_INVOLVED.X_TIN_TYPE_CD */
          /* A: EIN                                                                          */
          /* B: SSN/ITIN                                                                     */
          /* C: Foreign                                                                      */ 
          value $_tax_id_type 
          'E'='A'
          'S'='B'
          other=''
           ;
    run;

/* Create format to convert the country code from 3 characters to 2 chars          */
/* If there are 3-character country codes mapping to multiple 2-char country code, */
/* the first 2-char country code is used.                                          */

    data country_fmt;
                 set db_core.fsc_country_dim (keep=country_code_3 country_code_2 ) end=eof;
                 retain fmtname '_ctry_cd_3to2_fmt'
                        type    'C'
                 ;
                 length start end label $40; 
                 start=country_code_3;
                 end=start;
                 label=country_code_2;
                 if start='$' then label='';
                 output;
                 if eof then 
                 do;
                     start='**OTHER**';
                     end='';
                     hlo='O';
                     label='';
                     output;
                end; 
    run;

    proc sort data=country_fmt nodupkey;
    by start;
    run;


    proc format cntlin=country_fmt ;
    run;
     
%mend;
