/*----------------------------------------------------------------------------
 *    NAME: ctr_validation_job.sas
 *   TITLE: CTR validation job code
 * PRODUCT: SAS Currency Transaction Reporting
 *  SYSTEM: UNIX/Windows
 *----------------------------------------------------------------------------*/

/*-------------------------------------------------------------------
 * Copyright (c) 2015 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------
 *       This is a sample job script for running ctr_validate. 
 *       ctr_validate macro is shipped as ucmacro for demonstration only.  
 *---------------------------------------------------------------------------------------*/
 

%global job_rc trans_rc abortRC successYN; 

%let job_rc=0;
%let trans_rc=0;
%let abortRC=0;
%let successYN=Y;


/* Y to update report workflow when the report is validated as clean. */
/* N to not update report workflow.                                   */
%let ctr_update_workflowYN=N;

/* Y to run all web services in test mode.                            */
/* N to run web service to update data.                               */
/* This parameter is relevant only when ctr_update_workflowYN=Y       */
%let ctr_testYN=N; 

options mprint;



%macro ctr_validate_job;
 
/* Define the location and file name of the validation report. */

 %global printfile;
 /* Get the log file name and path */
 %let logfile=%sysfunc(getoption(LOG));
 /* If log file is specified by the job, replace .log to .htm to name the print file  */
 /* Otherwise, ctr_validate_job.htm as the print file name and put it logs directory. */
 %if "&logfile" ne "" %then  %let printfile=%sysfunc(tranwrd(&logfile,%str(.log),%str(.htm)));
 %else %let printfile=;

  /* Before calling ctr_validate, create a sas table with the key of the reports that should be validated */
  /* In the sample, all CTRX reports with ready_cd=D (draft) is included                                  */
  proc sql;
       create table rr_subset as
       select rr_rk
       from ecm_db.rr_live
       where rr_ready_cd='D' 
             and form_config_rk=212
          order by rr_rk
       ;
  quit;
 
  
  %if %ecm_nobs(rr_subset) > 0 %then
  %do; /* if rr_subset is not empty */
      /* call ctr_validate with the following parameters:                                    */
      /* update_workflowYN=N: report workflow is not updated even though the report is clean */
      /* subset_type=RR: input data set is report keys.                                      */
      /* subset_dsn=rr_subset: rr_subset is the input data set which contains record keys.   */
      /* skip_pivotYN=Y= transpose the data to the correct structure before validation.      */
      /* print_reportYN=Y: generate a validation report.                                     */
      /* testYN=N: do not run any web service calls as test mode.                            */
      /* stpYN=N: it is not a stored process call.                                           */                          
      %ctr_validate( update_workflowYN=&ctr_update_workflowYN
                     ,subset_type=RR
                     ,subset_dsn=rr_subset
                     ,skip_pivotYN=N
                     ,print_reportYN=Y
                     ,testYN=&ctr_testYN
                     ,stpYN=N 
                     );
   %end;
   %else 
   %do;
      %put NOTE: CTR: No CTR Data to validate. ;
      %put;
   %end;   
  
%mend;


%ctr_validate_job;
