/******************************************************************************/
/* This code may serve as a model for how you can validate data fields and    */ 
/* formatting in auto-generated CTR files, prior to submission.               */                  
/* The code provided is merely a sample; for your particular implementation,  */
/* code modifications may be needed and testing will be required.             */
/******************************************************************************/

%MACRO CTRX_RR_PIVOT;

                             
/*C301: Filing type is blank.*/
/* B1: The type of filing is not specified. */
error_cd="B1";
source_field_nm="X_CTR_FILING_TYPE_CD";
if X_CTR_FILING_TYPE_CD=""  then output;

/* F22: The element is not recorded. */
error_cd='F22';
source_field_nm="X_FILING_INST_TYPE_CTR_CD";
if X_FILING_INST_TYPE_CTR_CD=""  then output;

/* F24: The element contains a value that is not a valid code. */
error_cd="F24";
source_field_nm="X_FILING_INST_TYPE_CTR_CD";
if X_FILING_INST_TYPE_CTR_CD ne "" and index("ABCDZ",strip(X_FILING_INST_TYPE_CTR_CD))=0  then output;

/* F25: The element does not contain a value and Other is indicated. */
error_cd="F25";
source_field_nm="X_FILING_INST_TYPE_OTH_TXT";
if X_FILING_INST_TYPE_OTH_TXT="" and (X_FILING_INST_TYPE_CTR_CD  ='Z') then output;

/* F4: The element is recorded but without a value (i.e. the value is null). */
error_cd="F4";
source_field_nm="X_FILING_INST_PRIM_FED_REG_CD";
if (X_FILING_INST_PRIM_FED_REG_CD)=""  then output;

/* F5: The element contains a value that is not a valid code. */
error_cd="F5";
source_field_nm="X_FILING_INST_PRIM_FED_REG_CD";
if (X_FILING_INST_PRIM_FED_REG_CD) ne "" and index("ABCDEFG",strip(X_FILING_INST_PRIM_FED_REG_CD))=0  then output;

/* F27: The element is not recorded and the financial institution is Casino/Card club. */
error_cd="F27";
source_field_nm="X_FILING_INST_GAMING_TYPE_CD";
if X_FILING_INST_GAMING_TYPE_CD="" and (X_FILING_INST_TYPE_CTR_CD = 'A') then output;

/* F29: The element contains a value that is not a valid code. */
error_cd="F29";
source_field_nm="X_FILING_INST_GAMING_TYPE_CD";
if X_FILING_INST_GAMING_TYPE_CD ne "" and index("ABCZ",strip(X_FILING_INST_GAMING_TYPE_CD))=0 and (X_FILING_INST_TYPE_CTR_CD = 'A') then output;

/* F30: The element contains a value and the financial institution is not Casino/Card club. */
error_cd="F30";
source_field_nm="X_FILING_INST_GAMING_TYPE_CD";
if X_FILING_INST_GAMING_TYPE_CD ne "" and (X_FILING_INST_TYPE_CTR_CD ne 'A') then output;

/* F31: The element does not contain a value and Other is indicated. */
error_cd="F31";
source_field_nm="X_FILING_INST_GAMING_TYPE_OTH";
if X_FILING_INST_GAMING_TYPE_OTH="" and (X_FILING_INST_GAMING_TYPE_CD = 'Z') then output;

/* F32: The element is not recorded. */
error_cd="F32";
source_field_nm="X_FILING_INST_GAMING_TYPE_OTH";
if X_FILING_INST_GAMING_TYPE_OTH ne "" and (X_FILING_INST_GAMING_TYPE_CD ne 'Z') then output;

/* F7: The element is not recorded or does not contain a value. */
error_cd="F7";
source_field_nm="X_FILING_INST_NM";
if (X_FILING_INST_NM = "") then output; 

/* F9: The element is not recorded or does not contain a value.  */
error_cd="F9";
source_field_nm="X_FILING_INST_TIN_NO";
if X_FILING_INST_TIN_NO="" then output;

/* F11: The element contains a value that is not a valid code.  */
error_cd="F11";
source_field_nm="X_FILING_INST_TIN_NO";
if ((length(X_FILING_INST_TIN_NO) ne 9 or X_FILING_INST_TIN_NO in ('999999999' '000000000' '123456789') or notdigit(strip(X_FILING_INST_TIN_NO))))  then output;

/* F12: The element is not recorded or does not contain a value */
error_cd="F12";
source_field_nm="X_FILING_INST_ADDRESS_TXT";
if X_FILING_INST_ADDRESS_TXT="" then output;

/* F13: The element is not recorded or does not contain a value */
error_cd="F13";
source_field_nm="X_FILING_INST_CITY_NM";
if X_FILING_INST_CITY_NM="" then output;

/* F15: The element contains a value that is not a valid code. */
error_cd="F15";
source_field_nm="X_FILING_INST_STATE_CD";
if (put(X_FILING_INST_COUNTRY_CD||'_'||strip(X_FILING_INST_STATE_CD),$X_RT_STATE_PROVINCE.) ne 'VALID')  then output;

/* F14: The element is not recorded or does not contain a value. */
error_cd="F14";
source_field_nm="X_FILING_INST_STATE_CD";
if X_FILING_INST_STATE_CD=""  then output;

/* F20: The element contains a value that is not a valid code. */
error_cd="F20";
source_field_nm="X_FILING_INST_COUNTRY_CD";
if ((X_FILING_INST_COUNTRY_CD ne ' ' and X_FILING_INST_COUNTRY_CD not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/* F17: The element is not recorded or does not contain a value. */
error_cd="F17";
source_field_nm="X_FILING_INST_POSTAL_CD";
if X_FILING_INST_POSTAL_CD=""  then output;

/* F18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_cd="F18";
source_field_nm="X_FILING_INST_POSTAL_CD";
if (length(X_FILING_INST_POSTAL_CD) =9 and substr(X_FILING_INST_POSTAL_CD,6,4) in ('0000' '9999'))  then output;

/* F18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_cd="F18";
source_field_nm="X_FILING_INST_POSTAL_CD";
if (((length(X_FILING_INST_POSTAL_CD) ne 5 and length(X_FILING_INST_POSTAL_CD) ne 9)  or notdigit(strip(X_FILING_INST_POSTAL_CD)) > 0)) 
    and (X_FILING_INST_POSTAL_CD ne ' ') then output;

/*C302: Filing type contains an invalid code.*/
/* B4:The element contains a value not equal to "Y" or the element is null. */
error_cd="B4";
source_field_nm="X_CTR_FILING_TYPE_CD";
if X_CTR_FILING_TYPE_CD ne "" and index("ABC",strip(X_CTR_FILING_TYPE_CD))=0  then output;

/*C303: Prior report Document Control Number or BSA Identifier is blank.*/
/* B5: The element is not recorded and <CorrectsAmendsPriorReportIndicator> contains a "Y" value. */
error_cd="B5";
source_field_nm="X_PRIOR_AGENCY_REF_ID";
if X_PRIOR_AGENCY_REF_ID="" and (X_CTR_FILING_TYPE_CD not in ( 'A' ' ')) then output;

/*C304: Prior report Document Control Number or BSA Identifier contains non-numeric values.*/
/* B7: The value recorded for the element contains non-numeric characters. */
error_cd="B7";
source_field_nm="X_PRIOR_AGENCY_REF_ID";
if (X_PRIOR_AGENCY_REF_ID ne "" and (notdigit(strip(X_PRIOR_AGENCY_REF_ID)) > 0)) then output;

/*C307: Date of transaction is blank.*/
/* D1: The element is recorded but without a value (i.e. the value is null) */
error_cd="D1";
source_field_nm="X_TRANSACTION_DT";
if X_TRANSACTION_DT=.  then output;

/*C308: Date of transaction is greater than the date filed.*/
/* D3: The value recorded is greater than the date filed.*/
error_cd="D3";
source_field_nm="X_TRANSACTION_DT";
if (x_transaction_dt > today())  then output;

/*C309: Date of transaction is prior to January 1, 1950.*/
/* D4: The value recorded is prior to January 1, 1950. */
error_cd="D4";
source_field_nm="X_TRANSACTION_DT";
if (X_TRANSACTION_DT < mdy(1,1,1950)) and (X_TRANSACTION_DT ne .) then output;

/*C310: Date of transaction is not a numeric 8-position field in MMDDCCYY format where MM=month, DD=day,  CC=century, YY=year.*/
/* D2: The value recorded contains non-numeric characters or is in an invalid format.  */
error_cd="D2";
source_field_nm="X_TRANSACTION_DT";
if put(X_TRANSACTION_DT,MMDDYYN8.) = " "  then output;

/*C312: Total cash-in and total cash-out are blank.*/
/* D26: The total cash in and total cash out amounts for the FinCEN CTR are blank. (TotalCashOutAmountText) */
/* D12: The total cash in and total cash out amounts for the FinCEN CTR are blank. (totalCashInReceiveAmountText) */
error_cd="D12";
source_field_nm="X_US_CASH_IN_AMT";
if ( ((X_US_CASH_IN_AMT  <= 0) OR (X_US_CASH_IN_AMT =.))  and
     ((X_US_CASH_OUT_AMT <= 0) OR (X_US_CASH_OUT_AMT =.)) ) then output;

/*C313: Total cash-in contains non-numeric values or is in an invalid format.*/
/* D14: The value recorded for the element contains non-numeric characters */
error_cd="D14";
source_field_nm="X_US_CASH_IN_AMT";
if ((X_US_CASH_IN_AMT=0 or notdigit(put(X_US_CASH_IN_AMT,z15.)))) and (X_US_CASH_IN_AMT ne .) then output;

/*C314: Total cash-in Deposit(s) amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_A_AMT";
if ((X_US_CASH_IN_A_AMT=0 or notdigit(put(X_US_CASH_IN_A_AMT,z15.)))) and (X_US_CASH_IN_A_AMT ne .) then output;

/*C315: Total cash-in Payment(s) amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_B_AMT";
if ((X_US_CASH_IN_B_AMT=0 or notdigit(put(X_US_CASH_IN_B_AMT,z15.)))) and (X_US_CASH_IN_B_AMT ne .) then output;

/*C316: Total cash-in Currency received for fund transfer(s) out amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_C_AMT";
if ((X_US_CASH_IN_C_AMT=0 or notdigit(put(X_US_CASH_IN_C_AMT,z15.)))) and (X_US_CASH_IN_C_AMT ne .) then output;

/*C317: Total cash-in Purchase of negotiable instrument(s) amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_D_AMT";
if ((X_US_CASH_IN_D_AMT=0 or notdigit(put(X_US_CASH_IN_D_AMT,z15.)))) and (X_US_CASH_IN_D_AMT ne .) then output;

/*C318: Total cash-in Currency exchange(s) amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_E_AMT";
if ((X_US_CASH_IN_E_AMT=0 or notdigit(put(X_US_CASH_IN_E_AMT,z15.)))) and (X_US_CASH_IN_E_AMT ne .) then output;

/*C319: Total cash-in "Currency to prepaid access" amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_F_AMT";
if ((X_US_CASH_IN_F_AMT=0 or notdigit(put(X_US_CASH_IN_F_AMT,z15.)))) and (X_US_CASH_IN_F_AMT ne .) then output;

/*C320: Total cash-in Purchase(s) of casino chips, tokens, and other gaming instruments amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_G_AMT";
if ((X_US_CASH_IN_G_AMT=0 or notdigit(put(X_US_CASH_IN_G_AMT,z15.)))) and (X_US_CASH_IN_G_AMT ne .) then output;

/*C321: Total cash-in "Currency wager(s) including money plays" amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_H_AMT";
if ((X_US_CASH_IN_H_AMT=0 or notdigit(put(X_US_CASH_IN_H_AMT,z15.)))) and (X_US_CASH_IN_H_AMT ne .) then output;

/*C322: Total cash-in Bills inserted into gaming devices amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_I_AMT";
if ((X_US_CASH_IN_I_AMT=0 or notdigit(put(X_US_CASH_IN_I_AMT,z15.)))) and (X_US_CASH_IN_I_AMT ne .) then output;

/*C323: Total cash-in Other amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_US_CASH_IN_Z_AMT";
if ((X_US_CASH_IN_Z_AMT=0 or notdigit(put(X_US_CASH_IN_Z_AMT,z15.)))) and (X_US_CASH_IN_Z_AMT ne .) then output;

/*C324: Total cash-in Other description is blank.*/
/* D21: The element does not contain a value and Other is indicated. */
error_cd="D21";
source_field_nm="X_US_CASH_IN_OTH_TXT";
if X_US_CASH_IN_OTH_TXT="" and (X_US_CASH_IN_Z_AMT > 0) then output;

/*C327: Total cash-out contains non-numeric values or is in an invalid format.*/
/* D28: The value recorded for the element contains non-numeric characters. */
error_cd="D28";
source_field_nm="X_US_CASH_OUT_AMT";
if ((X_US_CASH_OUT_AMT=0 or notdigit(put(X_US_CASH_OUT_AMT,z15.)))) and (X_US_CASH_OUT_AMT ne .) then output;

/*C328: Total cash-out "Withdrawal(s)" amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_A_AMT";
if ((X_US_CASH_OUT_A_AMT=0 or notdigit(put(X_US_CASH_OUT_A_AMT,z15.)))) and (X_US_CASH_OUT_A_AMT ne .) then output;

/*C329: Total cash-out Advance(s) on credit (including markers) amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_B_AMT";
if ((X_US_CASH_OUT_B_AMT=0 or notdigit(put(X_US_CASH_OUT_B_AMT,z15.)))) and (X_US_CASH_OUT_B_AMT ne .) then output;

/*C330: Total cash-out "Currency paid from funds transfer(s) in" amount contains non-numeric values or is in  an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_C_AMT";
if ((X_US_CASH_OUT_C_AMT=0 or notdigit(put(X_US_CASH_OUT_C_AMT,z15.)))) and (X_US_CASH_OUT_C_AMT ne .) then output;

/*C331: Total cash-out Negotiable instrument(s) cashed amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_D_AMT";
if ((X_US_CASH_OUT_D_AMT=0 or notdigit(put(X_US_CASH_OUT_D_AMT,z15.)))) and (X_US_CASH_OUT_D_AMT ne .) then output;

/*C332: Total cash-out "Currency exchange(s)" amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_E_AMT";
if ((X_US_CASH_OUT_E_AMT=0 or notdigit(put(X_US_CASH_OUT_E_AMT,z15.)))) and (X_US_CASH_OUT_E_AMT ne .) then output;

/*C333: Total cash-out Currency to prepaid access amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_F_AMT";
if ((X_US_CASH_OUT_F_AMT=0 or notdigit(put(X_US_CASH_OUT_F_AMT,z15.)))) and (X_US_CASH_OUT_F_AMT ne .) then output;

/*C334: Total cash-out Redemption(s) of casino chips, tokens, TITO tickets, and other gaming instruments amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_G_AMT";
if ((X_US_CASH_OUT_G_AMT=0 or notdigit(put(X_US_CASH_OUT_G_AMT,z15.)))) and (X_US_CASH_OUT_G_AMT ne .) then output;

/*C335: Total cash-out Payment(s) on wager(s) (including race book and OTB or sports pool) amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_H_AMT";
if ((X_US_CASH_OUT_H_AMT=0 or notdigit(put(X_US_CASH_OUT_H_AMT,z15.)))) and (X_US_CASH_OUT_H_AMT ne .) then output;

/*C336: Total cash-out Travel and complimentary expenses and gaming incentives amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_I_AMT";
if ((X_US_CASH_OUT_I_AMT=0 or notdigit(put(X_US_CASH_OUT_I_AMT,z15.)))) and (X_US_CASH_OUT_I_AMT ne .) then output;

/*C337: Total cash-out Payment for tournament, contest or other promotions amount contains non-numeric values  or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_US_CASH_OUT_J_AMT";
if ((X_US_CASH_OUT_J_AMT=0 or notdigit(put(X_US_CASH_OUT_J_AMT,z15.)))) and (X_US_CASH_OUT_J_AMT ne .) then output;

/*C338: Total cash-out Other amount contains non-numeric values or is in an invalid format.*/
/* D28: The value recorded for the element contains non-numeric characters. */
error_cd="D28";
source_field_nm="X_US_CASH_OUT_Z_AMT";
if ((X_US_CASH_OUT_Z_AMT=0 or notdigit(put(X_US_CASH_OUT_Z_AMT,z15.)))) and (X_US_CASH_OUT_Z_AMT ne .) then output;

/*C339: Total cash-out Other description is blank.*/
/* D35: The element does not contain a value and Other is indicated. */
error_cd="D35";
source_field_nm="X_US_CASH_OUT_OTH_TXT";
if X_US_CASH_OUT_OTH_TXT="" and (X_US_CASH_OUT_Z_AMT > 0) then output;

/*C342: Total cash-in is not equal to the total amounts recorded in items 25a-i and 25z.*/
/* D15: The sum of all subtotal cash in amounts does not equal the total cash in amount recorded.*/
error_cd="D15";
source_field_nm="X_US_CASH_IN_AMT";
if (X_US_CASH_IN_AMT ne sum(X_US_CASH_IN_A_AMT,X_US_CASH_IN_B_AMT,X_US_CASH_IN_C_AMT,X_US_CASH_IN_D_AMT,X_US_CASH_IN_E_AMT,
                            X_US_CASH_IN_F_AMT,X_US_CASH_IN_G_AMT,X_US_CASH_IN_H_AMT,X_US_CASH_IN_I_AMT,X_US_CASH_IN_Z_AMT)) 
                            and (X_US_CASH_IN_AMT > 0) then output;

/*C343: Total cash-out is not equal to the total amounts recorded in items 27a-j and 27z.*/
/* D29: The sum of all subtotal cash out amounts does not equal the total cash out amount recorded. */
error_cd="D29";
source_field_nm="X_US_CASH_OUT_AMT";
if (X_US_CASH_OUT_AMT ne sum(X_US_CASH_OUT_A_AMT,X_US_CASH_OUT_B_AMT,X_US_CASH_OUT_C_AMT,X_US_CASH_OUT_D_AMT,X_US_CASH_OUT_E_AMT,
                             X_US_CASH_OUT_F_AMT,X_US_CASH_OUT_G_AMT,X_US_CASH_OUT_H_AMT,X_US_CASH_OUT_I_AMT,X_US_CASH_OUT_J_AMT,
                             X_US_CASH_OUT_Z_AMT)) and (X_US_CASH_OUT_AMT > 0) then output;

/*C344: Contact office is blank.*/
/* G3: The element is not recorded or does not contain a value.*/
error_cd="G3";
source_field_nm="X_CTR_CONTACT_OFFICE";
if X_CTR_CONTACT_OFFICE=""  then output;

/*C345: Contact telephone number is blank.*/
/* G5: The element is not recorded or does not contain a value (e.g. the value is null). */
error_cd="G5";
source_field_nm="X_CTR_CONTACT_PHONE_NO";
if X_CTR_CONTACT_PHONE_NO=""  then output;

/*C346: Contact telephone number is contains non-numeric values or is in an invalid format.*/
/* G4: The value recorded for the element contains non-numeric characters. */
error_cd="G4";
source_field_nm="X_CTR_CONTACT_PHONE_NO";
if (X_CTR_CONTACT_PHONE_NO ne "" and (notdigit(strip(X_CTR_CONTACT_PHONE_NO)) > 0))   then output;
 

/*C347: Date filed is blank.*/
/* F37: The element is recorded but without a value (i.e. the value is null).  */

/*C348: Date filed is not a numeric 8-position field in MMDDCCYY format where MM=month, DD=day, CC=century, YY=year.*/
/* F38: The value recorded for the element is not in the correct format (YYYYMMDD) */
error_cd="F38";
source_field_nm="X_FILED_DT";
if put(X_FILED_DT,YYMMDDN8.) = " "  then output;
 

/*C349: Date filed is greater than the current date or is less than 01/01/2000.*/

/*C350: Total cash-in and/or total cash-out contains an amount less than or equal to $10,000.*/
/* D13: The Total cash in and/or total cash out amounts for the FinCEN CTR contain an amount less than or equal to $10,000. (TotalCashInReceiveAmountText)*/
/* D27: The Total cash in and/or total cash out amounts for the FinCEN CTR contain an amount less than or equal to $10,000. (TotalCashOutAmountText)*/
error_cd="D13";
source_field_nm="X_US_CASH_IN_AMT";
if ((X_US_CASH_IN_AMT < 10000) and (X_US_CASH_IN_AMT ^= .))   then output;
error_cd="D27";
source_field_nm="X_US_CASH_OUT_AMT";
if ((X_US_CASH_OUT_AMT < 10000) AND (X_US_CASH_OUT_AMT ^= .))  then output;

%mend; /* CTRX_RR_PIVOT */


%MACRO CTRX_R_X_FOREIGN_CASH_IN;

/*C325: Foreign cash-in amount contains non-numeric values or is in an invalid format.*/
/* D16: The value recorded for the element contains non-numeric characters. */
error_cd="D16";
source_field_nm="X_FOREIGN_CASH_IN_AMT";
if ((X_FOREIGN_CASH_IN_AMT=0 or notdigit(put(X_FOREIGN_CASH_IN_AMT,z15.)))) and (X_FOREIGN_CASH_IN_AMT ne .) then output;

/*C326: Foreign cash-in country is not a valid country code.*/
/* D25: The element contains a value that is not a valid code. */
error_cd="D25";
source_field_nm="X_FOREIGN_CASH_IN_COUNTRY_CD";
if (put(X_FOREIGN_CASH_IN_COUNTRY_CD,$X_RT_COUNTRY.) ne 'VALID')  then output;

%mend; /* CTRX_R_X_FOREIGN_CASH_IN */


%MACRO CTRX_R_X_FOREIGN_CASH_OUT;

/*C340: Foreign cash-out amount contains non-numeric values or is in an invalid format.*/
/* D30: The value recorded for the element contains non-numeric characters. */
error_cd="D30";
source_field_nm="X_FOREIGN_CASH_OUT_AMT";
if ((X_FOREIGN_CASH_OUT_AMT=0 or notdigit(put(X_FOREIGN_CASH_OUT_AMT,z15.)))) and (X_FOREIGN_CASH_OUT_AMT ne .) then output;

/*C341: Foreign cash-out country is not a valid country code.*/
/* D39: The element contains a value that is not a valid code. */
error_cd="D39";
source_field_nm="X_FOREIGN_CASH_OUT_COUNTRY_CD";
if (put(X_FOREIGN_CASH_OUT_COUNTRY_CD,$X_RT_COUNTRY.) ne 'VALID')  then output;

%mend; /* CTRX_R_X_FOREIGN_CASH_OUT */


%MACRO CTRX_R_X_INSTITUTION;

/*C201: Financial institution primary federal regulator is blank.*/
/* E3: The element is not recorded.  */
error_cd="E3";
source_field_nm="X_INST_PRIM_FED_REG_CD";
if X_INST_PRIM_FED_REG_CD=""  then output;

/*C202: Financial institution primary federal regulator contains an invalid code.*/
/* E5: The element contains a value that is not a valid code. */
error_cd="E5";
source_field_nm="X_INST_PRIM_FED_REG_CD";
if X_INST_PRIM_FED_REG_CD ne "" and index("ABCDEFGZ",strip(X_INST_PRIM_FED_REG_CD))=0  then output;

/*C203: Financial institution legal name is blank.*/
/* E6: The element is not recorded or does not contain a value. */
error_cd="E6";
source_field_nm="X_INST_LEGAL_NM";
if X_INST_LEGAL_NM=""  then output;

/*C204: Financial institution EIN is blank.*/
/* E8: The element is not recorded or does not contain a value and unknown is not indicated.  */
error_cd="E8";
source_field_nm="X_INST_TIN_NO";
if X_INST_TIN_NO="" and X_INST_TIN_UNKNOWN_FLG ne "1" then output;

/*C205: Financial institution EIN is an invalid number string such as all zeros, all nines, or 123456789, or contains non-numeric values.*/
/* E7: The element contains an invalid number string for a US TIN.  */
error_cd="E7";
source_field_nm="X_INST_TIN_NO";
if ((length(X_INST_TIN_NO) ne 9 or X_INST_TIN_NO in ('999999999' '000000000' '123456789') or notdigit(strip(X_INST_TIN_NO))))  
   and (X_INST_TIN_UNKNOWN_FLG ne "1") then output;

/*C206: Financial institution street address is blank.*/
/* E12: The element is not recorded or does not contain a value. */
error_cd="E12";
source_field_nm="X_INST_ADDRESS_TXT";
if X_INST_ADDRESS_TXT=""  then output;

/*C207: Financial institution city is blank.*/
/* E13: The element is not recorded or does not contain a value. */
error_cd="E13";
source_field_nm="X_INST_CITY_NM";
if X_INST_CITY_NM=""  then output;

/*C208: Financial institution state is not a valid state code.*/
/* E15: The element contains a value that is not a valid code. */
error_cd="E15";
source_field_nm="X_INST_STATE_CD";
if (put(X_INST_COUNTRY_CD||'_'||strip(X_INST_STATE_CD),$X_RT_STATE_PROVINCE.) ne 'VALID')  then output;

/*C209: Financial institution state is blank.*/
/* E14: The element is not recorded or does not contain a value. */
error_cd="E14";
source_field_nm="X_INST_STATE_CD";
if X_INST_STATE_CD=""  then output;

/* E20: The element contains a value that is not a valid code. */
error_cd="E20";
source_field_nm="X_INST_COUNTRY_CD";
if ((X_INST_COUNTRY_CD ne ' ' and X_INST_COUNTRY_CD not in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/*C210: Financial institution ZIP Code is blank.*/
/* E17: The element is not recorded or does not contain a value. */
error_cd="E17";
source_field_nm="X_INST_POSTAL_CD";
if X_INST_POSTAL_CD=""  then output;

/*C211: Financial institution ZIP Code is a nine digit code ending in four zeros/nines.*/
/* E18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_cd="E18";
source_field_nm="X_INST_POSTAL_CD";
if (length(X_INST_POSTAL_CD) =9 and substr(x_inst_postal_cd,6,4) in ('0000' '9999'))  then output;

/*C212: Financial institution ZIP Code is not equal to five or nine digits, or contains non-numeric values.*/
/* E18: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_cd="E18";
source_field_nm="X_INST_POSTAL_CD";
if (((length(X_INST_POSTAL_CD) ne 5 and length(X_INST_POSTAL_CD) ne 9)  or notdigit(strip(X_INST_POSTAL_CD)) > 0)) and (X_INST_POSTAL_CD ne ' ') then output;

/*C213: Financial institution type is blank.*/
/* E23: The element is recorded but without a value (i.e. the value is null). */
error_cd="E23";
source_field_nm="X_INST_TYPE_CTR_CD";
if X_INST_TYPE_CTR_CD=""  then output;

/*C214: Financial institution type contains an invalid code.*/
/* E24: The element contains a value that is not a valid code. */
error_cd="E24";
source_field_nm="X_INST_TYPE_CTR_CD";
if X_INST_TYPE_CTR_CD ne "" and index("ABCDZ",strip(X_INST_TYPE_CTR_CD))=0  then output;

/*C215: Financial institution type other description is blank.*/
/* E25: The element does not contain a value and Other is indicated. */
error_cd="E25";
source_field_nm="X_INST_TYPE_OTH_TXT";
if X_INST_TYPE_OTH_TXT="" and (X_INST_TYPE_CTR_CD  ='Z') then output;

/*C216: Gaming institution type is blank.*/
/* E28: The element is recorded but without a value (i.e. the value is null). */
error_cd="E28";
source_field_nm="X_INST_GAMING_TYPE_CD";
if X_INST_GAMING_TYPE_CD="" and (X_INST_TYPE_CTR_CD = 'A') then output;

/*C217: Gaming institution type contains an invalid code.*/
/* E29: The element contains a value that is not a valid code. */
error_cd="E29";
source_field_nm="X_INST_GAMING_TYPE_CD";
if X_INST_GAMING_TYPE_CD ne "" and index("ABCZ",strip(X_INST_GAMING_TYPE_CD))=0 and (X_INST_TYPE_CTR_CD = 'A') then output;

/*C218: Gaming institution type other description is blank.*/
/* E31: The element does not contain a value and Other is indicated. */
error_cd="E31";
source_field_nm="X_INST_GAMING_TYPE_OTH_TXT";
if X_INST_GAMING_TYPE_OTH_TXT="" and (X_INST_GAMING_TYPE_CD = 'Z') then output;

/*C219: Financial institution identification type contains an invalid code.*/
/* E34: The element contains a value that is not a valid code. */
error_cd="E34";
source_field_nm="X_INST_ID_TYPE_CD";
if X_INST_ID_TYPE_CD ne "" and index("ABCDE",strip(X_INST_ID_TYPE_CD))=0  then output;

/*C220: Financial institution identification number is blank when one of boxes 39a-39e has been checked.*/
/* E35: The element is not recorded or does not contain a value. */
error_cd="E35";
source_field_nm="X_INST_ID";
if X_INST_ID="" and (X_INST_ID_TYPE_CD ne '') then output;

/* E36: The cash-in and cash-out amounts for the transaction location are blank. */
error_cd="E36";
source_field_nm="X_CASH_IN_AMT";
if ((X_CASH_IN_AMT=0) or (X_CASH_IN_AMT=.)) and ((X_CASH_OUT_AMT=0) or (X_CASH_OUT_AMT=.)) then output;


%mend; /* CTRX_R_X_INSTITUTION */


%MACRO CTRX_R_X_PERSON_INVOLVED;

/*C401: Person last/legal name is blank.*/
/* C8: The element is not recorded or does not contain a value, <PartyNameTypeCode> is "L" (Legal), and unknown is not indicated. */
error_cd="C8";
source_field_nm="X_LAST_NM";
if X_LAST_NM="" and (X_LAST_NM_UNKNOWN_FLG NE "1") then output;

/*C402: Person first name is blank.*/
/* C11: The element is not recorded or does not contain a value for an individual and unknown is not indicated.  */
error_cd="C11";
source_field_nm="X_FIRST_NM";
if X_FIRST_NM="" and (X_FIRST_NM_UNKNOWN_FLG NE "1" and X_ENTITY_FLG ne "1") then output;

/*C403: Person gender contains an invalid code.*/
/* C13: The person is an individual and gender is not specified in FemaleGenderIndicator, MaleGenderIndicator, or UnknownGenderIndicator. */
error_cd="C13";
source_field_nm="X_GENDER_CD";
if index("ABC",strip(X_GENDER_CD))=0 and (X_ENTITY_FLG ne "1") then output;

/*C404: Person street address is blank.*/
/* C16: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C16";
source_field_nm="X_ADDRESS_TXT";
if X_ADDRESS_TXT="" and (X_ADDRESS_UNKNOWN_FLG NE "1") then output;

/*C405: Person city is blank.*/
/* C18: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C18";
source_field_nm="X_CITY_NM";
if X_CITY_NM="" and (X_CITY_UNKNOWN_FLG NE "1") then output;

/*C406: Person state is not a valid state code and person country is equal to CA, MX, or US.*/
/* C21: The element contains a value that is not a valid code. */
error_cd="C21";
source_field_nm="X_STATE_CD";
if (put(strip(X_COUNTRY_CD)||'_'||strip(X_STATE_CD),$X_RT_STATE_PROVINCE.) ne 'VALID') and
    ((X_STATE_CD ne ' ' and X_COUNTRY_CD in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR'))) then output;

/*C407: Person state is blank and person country is equal to CA, MX, or US.*/
/* C20: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C20";
source_field_nm="X_STATE_CD";
if X_STATE_CD="" and (X_STATE_UNKNOWN_FLG NE "1" and X_COUNTRY_CD in ('CA' 'MX' 'US' 'AS' 'GU' 'MH' 'FM' 'MP' 'PW' 'PR')) then output;

/*C408: Person ZIP Code or foreign postal code is blank.*/
/* C25: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C25";
source_field_nm="X_POSTAL_CD";
if X_POSTAL_CD="" and (X_POSTAL_UNKNOWN_FLG NE "1") then output;

/*C409: Person ZIP Code is a nine digit code ending in four zeros/nines and person country is equal to US.*/
/* C26: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_cd="C26";
source_field_nm="X_POSTAL_CD";
if ((LENGTH(X_POSTAL_CD)=9 AND SUBSTRN(X_POSTAL_CD,6,4) IN  ('0000' '9999' ))) and ((X_POSTAL_UNKNOWN_FLG ne "1" and X_COUNTRY_CD = 'US')) then output;

/*C410: Person ZIP Code is not equal to five or nine digits or contains non-numeric values and person country is  equal to US.*/
/* C26: The value recorded contains non- numeric characters or is in an invalid format and the country is US or a U.S. Territory. */
error_cd="C26";
source_field_nm="X_POSTAL_CD";
if (((length(X_POSTAL_CD) ne 5 and length(X_POSTAL_CD) ne 9)  or notdigit(strip(X_POSTAL_CD)))) and ((X_POSTAL_UNKNOWN_FLG NE "1" and X_COUNTRY_CD='US')) then output;

/*C411: Person country is blank.*/
/* C28: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C28";
source_field_nm="X_COUNTRY_CD";
if X_COUNTRY_CD="" and (X_COUNTRY_UNKNOWN_FLG NE "1") then output;

/*C412: Person country is not a valid country code.*/
/* C29: The element contains a value that is not a valid code. */
error_cd="C29";
source_field_nm="X_COUNTRY_CD";
if (put(X_COUNTRY_CD,$X_RT_COUNTRY.) ne 'VALID') and (X_COUNTRY_UNKNOWN_FLG NE "1") then output;

/*C413: Person TIN is blank.*/
/* C32: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C32";
source_field_nm="X_TIN_NO";
if X_TIN_NO="" and (X_TIN_UNKNOWN_FLG ne "1") then output;

/*C414: Person TIN is an invalid number string such as all zeros, all nines, 123456789, or a U.S. TIN and not nine digits.*/
/* C33: The element contains an invalid number string for a US TIN.  */
error_cd="C33";
source_field_nm="X_TIN_NO";
if ((length(X_TIN_NO) ne 9 or X_TIN_NO in ('999999999' '000000000' '123456789') or notdigit(strip(X_TIN_NO)))) and (X_TIN_NO ne ' ') 
   and (X_TIN_UNKNOWN_FLG ne "1") then output;

/*C415: Person TIN type is blank.*/
/* C32: The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C32";
source_field_nm="X_TIN_NO";
if X_TIN_NO="" and (X_TIN_UNKNOWN_FLG ne "1") then output;

/*C416: Person TIN type contains an invalid code.*/
/* C36: The element contains a value that is not a valid code. */
error_cd="C36";
source_field_nm="X_TIN_TYPE_CD";
if X_TIN_TYPE_CD ne "" and index("ABC",strip(X_TIN_TYPE_CD))=0 and (X_TIN_TYPE_CD ne ' ') then output;

/*C417: Person date of birth is blank.*/
/* C37: The element is not recorded for an individual and unknown is not indicated. */
error_cd="C37";
source_field_nm="X_BIRTHDAY_TXT";
if X_BIRTHDAY_TXT="" and (X_BIRTH_DT_UNKNOWN_FLG NE "1" and X_ENTITY_FLG ne "1") then output;

/*C418: Person date of birth is less than January 1, 1900.*/
/* C39: The date of birth is less than January 1, 1900 or is greater than the date of transaction.*/
error_cd="C39";
source_field_nm="X_BIRTHDAY_TXT";
if (SUBSTR(X_BIRTHDAY_TXT,5,4) < '1900') and (X_BIRTHDAY_TXT ne '') then output;
/**** SKIP *****/

/*C419: Person date of birth contains non-numeric values or is in an invalid format.*/
/* C40: The value recorded for the element is not in the correct format (YYYYMMDD) or is not a valid date. */

/*C420: Person telephone number contains non-numeric values or is in an invalid format.*/
/* C42: The value recorded for the element contains non-numeric characters or is an invalid number string. */
error_cd="C42";
source_field_nm="X_PHONE_NO";
if (X_PHONE_NO ne "" and (notdigit(strip(X_PHONE_NO)) > 0))   then output;
/**** SKIP *****/

/*C421: Person e-mail address is in an invalid format.*/
/* C43: The value recorded for the element is not in the correct format for an email address. */

/*C422: Person identification is blank.*/
/* C45: The element is recorded but without a value (i.e. the value is null). */
error_cd="C45";
source_field_nm="X_PERSONAL_ID_TYPE_CD";
if X_PERSONAL_ID_TYPE_CD=""  then output;

/*C423: Person identification type contains an invalid code.*/
/* C46: The element contains a value that is not a valid code. */
error_cd="C46";
source_field_nm="X_PERSONAL_ID_TYPE_CD";
if X_PERSONAL_ID_TYPE_CD ne "" and index("ABCDZ",strip(X_PERSONAL_ID_TYPE_CD))=0  then output;

/*C424: Person identification other description is blank.*/
/* C48: The element is not recorded or does not contain a value and Other is indicated. */
error_cd="C48";
source_field_nm="X_PERSONAL_ID_TYPE_OTH_TXT";
if X_PERSONAL_ID_TYPE_OTH_TXT="" and (X_PERSONAL_ID_TYPE_CD='Z') then output;

/*C425: Person identification issuing state is not a valid state code and person country is equal to CA,  MX, or US.*/
/* C51: The element contains a value that is not a valid code. */
error_cd="C51";
source_field_nm="X_ID_AUTHORITY_STATE_CD";
if (put(strip(X_ID_AUTHORITY_COUNTRY_CD)||'_'||strip(X_ID_AUTHORITY_STATE_CD),$X_RT_STATE_PROVINCE.) ne 'VALID') and (X_ID_AUTHORITY_COUNTRY_CD IN ('US' 'MX' 'CA') and X_ID_AUTHORITY_STATE_CD ne ' ') then output;

/*C426: Person identification issuing country is not a valid country code.*/
/* C54: The element contains a value that is not a valid code. */
error_cd="C54";
source_field_nm="X_ID_AUTHORITY_COUNTRY_CD";
if (put(X_ID_AUTHORITY_COUNTRY_CD,$X_RT_COUNTRY.) ne 'VALID') and (X_ID_AUTHORITY_COUNTRY_CD  > '') then output;

/*C427: Cash-in amount for the person involved in the transaction contains non-numeric values or is in an invalid format.*/
/* C59: The value recorded for the element contains non-numeric characters. */
error_cd="C59";
source_field_nm="X_CASH_IN_AMT";
if ((X_CASH_IN_AMT=0 or notdigit(put(X_CASH_IN_AMT,z15.)))) and (X_CASH_IN_AMT ne .) then output;

/*C428: Cash-out amount for the person involved in the transaction contains non-numeric values or is in an invalid format.*/
/* C61: The value recorded for the element contains non-numeric characters. */
error_cd="C61";
source_field_nm="X_CASH_OUT_AMT";
if ((X_CASH_OUT_AMT=0 or notdigit(put(X_CASH_OUT_AMT,z15.)))) and (X_CASH_OUT_AMT ne .) then output;

/* C58: The cash-in and cash-out amounts transacted by or for the person involved are blanked. */
error_cd="C58";
source_field_nm="X_CASH_OUT_AMT";
if ( ((X_CASH_IN_AMT=0) or (X_CASH_IN_AMT=.)) AND ((X_CASH_OUT_AMT=0) or (X_CASH_OUT_AMT=.)) ) then output;

/*C429: Person identification number is blank.*/
/* C56 The element is not recorded or does not contain a value and unknown is not indicated. */
error_cd="C56";
source_field_nm="X_ID_NO";
if X_ID_NO="" and (X_PERSONAL_ID_TYPE_CD NE 'A') then output;

/* C1: A required party associated with either the person on whose behalf the transaction was conducted  */
/*     or the person conducting transaction on own behalf is missing.                                    */
error_cd="C1";
source_field_nm="X_PERSON_INVOLVED_TYPE_CD";
if index("ABCD",strip(X_PERSON_INVOLVED_TYPE_CD))=0  then output;

%mend; /* CTRX_R_X_PERSON_INVOLVED */


%MACRO CTRX_R_X_TRAN_METHOD;

/*C311: Method of transaction(s) contains an invalid code.*/
/* D5: The element contains a value not equal to “Y” or the element is null. */
error_cd="D5";
source_field_nm="X_TRAN_METHOD_TYPE_CD";
if X_TRAN_METHOD_TYPE_CD ne "" and index("ABCDEF",strip(X_TRAN_METHOD_TYPE_CD))=0  then output;

%mend; /* CTRX_R_X_TRAN_METHOD */


