/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    pga_calculate_entity_history.sas
| PRODUCT: SAS Peer Group Analysis
| SYSTEM:  Windows, Unix
| DESC:    Entity history attribute calculations for Peer Group Analysis.
+------------------------------------------------------------------------------+
| COMMENT: This code is submitted to LASR.
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------
 *  Host and Site specific customizations may be REQUIRED
 *---------------------------------------------------------*/
length sufficiency $5;
length history_period_code $1;
length segment_id $128;

 
  array date_key_array[&pga_period_sufficiency_count] _temporary_ ;

  if __first_in_partition  then do;

     history_begin_date_key = &pga_entity_hist_month_start;
     history_end_date_key = &pga_entity_hist_month_end;

     count = 0;

     trans_count_sum = 0;
     trans_amount_sum = 0;
     max_trans_amount_sum = 0;

     trans_count_uss = 0;
     trans_amount_uss = 0;
     max_trans_amount_uss = 0;

     max_trans_count = 0;
     trans_amount_max = 0;  
     max_max_trans_amount = 0;

     log10_trans_count_sum = 0;
     log10_trans_amount_sum = 0;
     log10_max_trans_amount_sum = 0;

     log10_trans_count_uss = 0;
     log10_trans_amount_uss = 0;
     log10_max_trans_amount_uss = 0;

     max_log10_trans_count = 0;
     max_log10_trans_amount = 0;
     max_log10_max_trans_amount = 0;

     do i = 1 to &pga_period_sufficiency_count;
       date_key_array[i] = 0;
     end;
     date_key_count = 1;
     date_key_array[date_key_count] = summary_begin_date_key;

     RunAsOfDate = input(put(&pga_entity_hist_month_end, 8.), yymmdd8.);
     entity_date =  input(&pga_current_entity_type._entity_date, yymmdd10.);
     nbrMonthsAsCust = intck('month',  ENTITY_DATE, RunAsOfDate) + 1; 
     entity_history_period_count = min(&pga_entity_hist_period_count , nbrMonthsasCust);

  /* -------------------------------------------------------------------------------------------------
   *  When rare error dates are pulled from the dim tables, this can create a negative value for 
   *  entity_history_period_count, which cause division by a negative number throughout all the 
   *  derived fields. The default correction is to set entity_history_period_count to 
   *  &pga_entity_hist_period_count. This code can be altered to meet customer's desired correction.
   * -------------------------------------------------------------------------------------------------*/
    if entity_history_period_count < 0 then entity_history_period_count = &pga_entity_hist_period_count; 
    
  end;
  else do;
     found_date = 0;
     do i = 1 to date_key_count;
       if summary_begin_date_key eq date_key_array[i] then do;
         found_date = 1;
         goto found_date_found;
       end;
     end;
found_date_found:;
     if found_date eq 0 and date_key_count < &pga_period_sufficiency_count then do;
       date_key_count + 1;  
       date_key_array[date_key_count] = summary_begin_date_key;
     end;
  end;
  history_period_code    = summary_period_code;
  history_period_count   = entity_history_period_count;
  period_sufficiency_count = &pga_period_sufficiency_count;
  segment_id = '&segKCDBUser';

  count + 1;

  trans_count_sum        + trans_count;
  trans_amount_sum       + trans_amount;
  max_trans_amount_sum   + max_trans_amount;

  trans_count_uss        = trans_count_uss + trans_count * trans_count;
  trans_amount_uss       = trans_amount_uss + trans_amount * trans_amount;
  max_trans_amount_uss   = max_trans_amount_uss + max_trans_amount * max_trans_amount;

  max_trans_count        = max(max_trans_count, trans_count);
  trans_amount_max       = max(trans_amount_max, trans_amount); 
  max_max_trans_amount   = max(max_max_trans_amount, max_trans_amount);

  log10_trans_count_sum        + log10_trans_count;
  log10_trans_amount_sum       + log10_trans_amount;
  log10_max_trans_amount_sum   + log10_max_trans_amount;

  log10_trans_count_uss        = log10_trans_count_uss + log10_trans_count * log10_trans_count;
  log10_trans_amount_uss       = log10_trans_amount_uss + log10_trans_amount * log10_trans_amount;
  log10_max_trans_amount_uss   = log10_max_trans_amount_uss + log10_max_trans_amount * log10_max_trans_amount;

  max_log10_trans_count        = max(max_log10_trans_count, log10_trans_count);
  max_log10_trans_amount       = max(max_log10_trans_amount, log10_trans_amount);
  max_log10_max_trans_amount   = max(max_log10_max_trans_amount, log10_max_trans_amount);


  if __last_in_partition then do;
     avg_trans_count        = trans_count_sum / entity_history_period_count;
     avg_trans_amount       = trans_amount_sum / entity_history_period_count;
     avg_max_trans_amount   = max_trans_amount_sum / entity_history_period_count;

     avg_log10_trans_count        = log10_trans_count_sum / entity_history_period_count;
     avg_log10_trans_amount       = log10_trans_amount_sum / entity_history_period_count;
     avg_log10_max_trans_amount   = log10_max_trans_amount_sum / entity_history_period_count;

     if entity_history_period_count > 1 then do;
     std_trans_count =
        sqrt((trans_count_uss - trans_count_sum * trans_count_sum / entity_history_period_count)/
             (entity_history_period_count - 1));
     std_trans_amount =
        sqrt((trans_amount_uss - trans_amount_sum * trans_amount_sum / entity_history_period_count)/
             (entity_history_period_count - 1));
     std_max_trans_amount =
        sqrt((max_trans_amount_uss - max_trans_amount_sum * max_trans_amount_sum / entity_history_period_count)/
             (entity_history_period_count - 1));

     std_log10_trans_count =
        sqrt((log10_trans_count_uss - log10_trans_count_sum * log10_trans_count_sum / entity_history_period_count)/
             (entity_history_period_count - 1));
     std_log10_trans_amount =
        sqrt((log10_trans_amount_uss - log10_trans_amount_sum * log10_trans_amount_sum / entity_history_period_count)/
             (entity_history_period_count - 1));
     std_log10_max_trans_amount =
        sqrt((log10_max_trans_amount_uss - log10_max_trans_amount_sum * log10_max_trans_amount_sum / entity_history_period_count)/
             (entity_history_period_count - 1));
     end;
     else do;
          std_trans_count = .;
          std_trans_amount = .;
          std_max_trans_amount = .;
          std_log10_trans_count = .;
          std_log10_trans_amount = .;
          std_log10_max_trans_amount = .;
     end;
     period_count = date_key_count;

     if date_key_count ge &pga_period_sufficiency_count then do;
       sufficiency = 'True';
     end;
     else do;
       sufficiency = 'False';
     end;
     __lasr_output = 1;
end; 
