%macro fcf_ce_load_alert_tx_agg;

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

     %let trans_rc = 0;
     %let _fcfmsg_=;

    /* Create temporary tables for alert transactions C/D/I and primary medium aggregates  */
    %put NOTE: FCF: Creating temporary tables for alert transactions C/D/I and primary medium aggregates;
    proc sql noprint;
        create table alert_cdi_agg_temp as select alert_id,
                                                  transaction_cdi_desc as group_code,
                                                  'CDI' as aggregate_type_code,
                                                  sum(transaction_amt) as aggregate_amt
                                          from seg_kc.fbu_customer_transaction
                                          where triggering_indicator_cd='Y'
                                          group by alert_id,transaction_cdi_desc
                                          order by alert_id, transaction_cdi_desc;
        create table alert_pri_agg_temp as select alert_id,
                                                  primary_medium_desc as group_code,
                                                  'PRI' as aggregate_type_code,
                                                  sum(transaction_amt) as aggregate_amt
                                          from seg_kc.fbu_customer_transaction
                                          where triggering_indicator_cd='Y'
                                          group by alert_id,primary_medium_desc
                                          order by alert_id, primary_medium_desc;
    quit;

    %fcf_rcset(&sqlrc);
        %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
           %let _fcfmsg_=ERROR: FCF: Failed to create temporary tables for alert transactions C/D/I and primary medium aggregates;
           %fcf_put_error;
           %goto MACRO_END;
        %end;

     %let obs_count = %fcf_get_observation_count(table=work.alert_cdi_agg_temp,SASDATASET=Y);
     %if &obs_count < 1 %then %do;
         %goto MACRO_END;
     %end;

     %let obs_count = %fcf_get_observation_count(table=work.alert_pri_agg_temp,SASDATASET=Y);
      %if &obs_count < 1 %then %do;
          %goto MACRO_END;
      %end;

    /* Merge alert_pri_agg_temp2 and alert_cdi_agg_temp2 datasets */
    %put NOTE: FCF: Merging alert_pri_agg_temp and alert_cdi_agg_temp datasets;
    %let now = %sysfunc(datetime());
    data alert_agg_temp;
        if 1 eq 0 then set seg_kc.fbu_alert_tx_agg;
        fbu_alert_tx_agg_pk =_N_;
        set alert_pri_agg_temp alert_cdi_agg_temp;
        created_timestamp=&now;
        group_desc=group_code;
    run;

    %fcf_rcset(&syserr);
        %if &trans_rc. eq 3 or &trans_rc. ge 5 %then %do;
           %let _fcfmsg_=ERROR: FCF: Failed to merge alert_pri_agg_temp2 and alert_cdi_agg_temp2 datasets. ;
           %fcf_put_error;
           %goto MACRO_END;
        %end;

    %let obs_count = %fcf_get_observation_count(table=work.alert_agg_temp,SASDATASET=Y);
         %if &obs_count < 1 %then %do;
             %goto MACRO_END;
         %end;

    /* Load the fbu_alert_tx_agg table */
    %put NOTE: FCF: Loading the fsk_alert_tx_agg table;
    proc append base=seg_kc.fbu_alert_tx_agg data=alert_agg_temp;
    run;

    %fcf_rcset(&syserr);
      %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
          %let _fcfmsg_=ERROR: FCF: Unable to load work.alert_agg_temp into seg_kc.fsk_alert_tx_agg table. ;
          %fcf_put_error;
          %goto MACRO_END;
      %end;


    /* Clean-up: Delete temporary alert transactions aggregate datasets */
    %put NOTE: FCF: Deleting temporary alert transactions aggregate datasets;
    proc datasets library=work nolist;
        delete alert_cdi_agg_temp;
        delete alert_pri_agg_temp;
        delete alert_agg_temp;
    quit;

    %MACRO_END:

       %if &trans_rc eq 3 or &trans_rc ge 5 %then
          %fcf_execute_macro(fcf_error);
       %else %do;
          %fcf_execute_macro(fcf_success);
       %end;
%mend fcf_ce_load_alert_tx_agg;
