/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_cdd_rel_control.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Case and Incident creation and synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_cdd_rel_control
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/
 
%macro fcf_iem_cdd_rel_ctrl;

    /*------------------------------------------
    * Set error code to 0 and message to blank
    *------------------------------------------*/
    %let trans_rc = 0;
    %let _fcfmsg_=;

    /*==========================================================================* 
     * Step:            Get all current FCF parties                             * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    FSC_PARTY_DIM - &db_core_libname..FSC_PARTY_DIM         * 
     * Target Table:    Current FCF parties -                                   * 
     *                   work.cur_fcf_party                                     * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(&db_core_libname..FSC_PARTY_DIM); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete cur_fcf_party;
    quit;

    proc sql;
       create table work.cur_fcf_party as
          select
             PARTY_KEY,
             SEGMENT_ID,
             PARTY_NUMBER,
             PARTY_TYPE_DESC,
             PARTY_TAX_ID,
             PARTY_NAME
       from &SOURCEDS
          where CHANGE_CURRENT_IND = 'Y'
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create table from Core;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let SOURCEDS = work.cur_fcf_party;

    /*==========================================================================* 
     * Step:            Get primary party FCF info                              * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   FSC_PARTY_ASSOC -                                       * 
     *                   &db_core_libname..FSC_PARTY_ASSOC                      * 
     *                  Current FCF parties -                                   * 
     *                   work.cur_fcf_party                                     * 
     * Target Table:    Primary (FCF) parties -                                 * 
     *                   work.prim_party                                        * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete prim_party;
    quit;

    proc sql;
       create table work.prim_party as
       select
          cur_fcf_party.PARTY_KEY,
          FSC_PARTY_ASSOC.SEGMENT_ID,
          FSC_PARTY_ASSOC.PARTY_NUMBER,
          cur_fcf_party.PARTY_NAME,
          cur_fcf_party.PARTY_TYPE_DESC,
          cur_fcf_party.PARTY_TAX_ID,
          FSC_PARTY_ASSOC.RELATED_PARTY_NUMBER,
          FSC_PARTY_ASSOC.RELATIONSHIP_TYPE_CODE
       from
          &db_core_libname..FSC_PARTY_ASSOC left join 
          work.cur_fcf_party
             on
             (
                FSC_PARTY_ASSOC.SEGMENT_ID = cur_fcf_party.SEGMENT_ID
                and FSC_PARTY_ASSOC.PARTY_NUMBER = cur_fcf_party.PARTY_NUMBER
             )
       where
          FSC_PARTY_ASSOC.CHANGE_CURRENT_IND = 'Y'
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create table from Core;
        %fcf_put_error;
        %goto MACRO_END;
    %end;
    
    /*==========================================================================* 
     * Step:            Get related party FCF info                              * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Primary (FCF) parties -                                 * 
     *                   work.prim_party                                        * 
     *                  Current FCF parties -                                   * 
     *                   work.cur_fcf_party                                     * 
     * Target Table:    Primary (FCF) and related (FCF)                         * 
     *                   parties - work.prim_rel_party                          * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete prim_rel_party;
    quit;

    proc sql;
       create table work.prim_rel_party as
       select
          prim_party.PARTY_KEY,
          prim_party.SEGMENT_ID,
          prim_party.PARTY_NUMBER,
          prim_party.PARTY_NAME,
          prim_party.PARTY_TYPE_DESC,
          prim_party.PARTY_TAX_ID,
          prim_party.RELATIONSHIP_TYPE_CODE,
          prim_party.RELATED_PARTY_NUMBER,
          cur_fcf_party.PARTY_KEY as RELATED_PARTY_KEY,
          cur_fcf_party.PARTY_TYPE_DESC as RELATED_PARTY_TYPE_DESC,
          cur_fcf_party.PARTY_TAX_ID as RELATED_PARTY_TAX_ID,
          cur_fcf_party.PARTY_NAME as RELATED_PARTY_NAME
       from
          work.prim_party left join 
          work.cur_fcf_party
             on
             (
                prim_party.RELATED_PARTY_NUMBER = cur_fcf_party.PARTY_NUMBER
                and prim_party.SEGMENT_ID = cur_fcf_party.SEGMENT_ID
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to perform join;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Get all current FCF PTY Subs                            * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    PARTY_LIVE - ecm_db.PARTY_LIVE                          * 
     * Target Table:    Current ECM parties -                                   * 
     *                   work.cur_ecm_pty                                       * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(ecm_db.PARTY_LIVE); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete cur_ecm_pty;
    quit;

    proc sql;
       create table work.cur_ecm_pty as
          select
             PARTY_RK,
             (SUBSTRN(PARTY_ID,5)) as PARTY_ID,
             SOURCE_SYSTEM_CD,
             PARTY_TYPE_CD,
             PARTY_CATEGORY_CD,
             PARTY_SUBCATEGORY_CD,
             PARTY_FULL_NM
       from &SOURCEDS
          where PARTY_TYPE_CD = 'FCFSUB'  & PARTY_CATEGORY_CD = 'PTY'
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create table from ECM;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let SOURCEDS = work.cur_ecm_pty;

    /*==========================================================================* 
     * Step:            Get primary party ECM info                              * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Primary (FCF) and related (FCF)                         * 
     *                   parties - work.prim_rel_party                          * 
     *                  Current ECM parties -                                   * 
     *                   work.cur_ecm_pty                                       * 
     * Target Table:    Primary (FCF/ECM) and related (FCF)                     * 
     *                   parties - work.prim_ecm_rel_pty                        * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete prim_ecm_rel_pty;
    quit;

    proc sql;
       create table work.prim_ecm_rel_pty as
       select
          prim_rel_party.PARTY_KEY,
          prim_rel_party.SEGMENT_ID,
          prim_rel_party.PARTY_NUMBER,
          prim_rel_party.PARTY_NAME,
          prim_rel_party.PARTY_TYPE_DESC,
          prim_rel_party.PARTY_TAX_ID,
          prim_rel_party.RELATIONSHIP_TYPE_CODE,
          prim_rel_party.RELATED_PARTY_NUMBER,
          prim_rel_party.RELATED_PARTY_KEY,
          prim_rel_party.RELATED_PARTY_TYPE_DESC,
          prim_rel_party.RELATED_PARTY_TAX_ID,
          prim_rel_party.RELATED_PARTY_NAME,
          cur_ecm_pty.PARTY_RK,
          cur_ecm_pty.SOURCE_SYSTEM_CD,
          cur_ecm_pty.PARTY_TYPE_CD,
          cur_ecm_pty.PARTY_CATEGORY_CD,
          cur_ecm_pty.PARTY_SUBCATEGORY_CD,
          cur_ecm_pty.PARTY_FULL_NM
       from
          work.prim_rel_party left join 
          work.cur_ecm_pty
             on
             (
                prim_rel_party.PARTY_NUMBER = cur_ecm_pty.PARTY_ID
                and prim_rel_party.SEGMENT_ID = cur_ecm_pty.SOURCE_SYSTEM_CD
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to perform join;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Get related party ECM info                              * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Primary (FCF/ECM) and related (FCF)                     * 
     *                   parties - work.prim_ecm_rel_pty                        * 
     *                  Current ECM parties -                                   * 
     *                   work.cur_ecm_pty                                       * 
     * Target Table:    Primary (FCF/ECM) and related                           * 
     *                  (FCF/ECM) parties -                                     * 
     *                   work.prim_ecm_rel_ecm_pty                              * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete prim_ecm_rel_ecm_pty;
    quit;

    proc sql;
       create table work.prim_ecm_rel_ecm_pty as
       select
          prim_ecm_rel_pty.PARTY_KEY,
          prim_ecm_rel_pty.SEGMENT_ID,
          prim_ecm_rel_pty.PARTY_NUMBER,
          prim_ecm_rel_pty.PARTY_NAME,
          prim_ecm_rel_pty.PARTY_TYPE_DESC,
          prim_ecm_rel_pty.PARTY_TAX_ID,
          prim_ecm_rel_pty.RELATIONSHIP_TYPE_CODE,
          prim_ecm_rel_pty.RELATED_PARTY_NUMBER,
          prim_ecm_rel_pty.RELATED_PARTY_KEY,
          prim_ecm_rel_pty.RELATED_PARTY_TYPE_DESC,
          prim_ecm_rel_pty.RELATED_PARTY_TAX_ID,
          prim_ecm_rel_pty.RELATED_PARTY_NAME,
          prim_ecm_rel_pty.PARTY_RK,
          prim_ecm_rel_pty.SOURCE_SYSTEM_CD,
          prim_ecm_rel_pty.PARTY_TYPE_CD,
          prim_ecm_rel_pty.PARTY_CATEGORY_CD,
          prim_ecm_rel_pty.PARTY_SUBCATEGORY_CD,
          prim_ecm_rel_pty.PARTY_FULL_NM,
          cur_ecm_pty.PARTY_RK as RELATED_PARTY_RK,
          cur_ecm_pty.SOURCE_SYSTEM_CD as RELATED_SOURCE_SYSTEM_CD,
          cur_ecm_pty.PARTY_TYPE_CD as RELATED_PARTY_TYPE_CD,
          cur_ecm_pty.PARTY_CATEGORY_CD as RELATED_PARTY_CATEGORY_CD,
          cur_ecm_pty.PARTY_SUBCATEGORY_CD as RELATED_PARTY_SUBCATEGORY_CD,
          cur_ecm_pty.PARTY_FULL_NM as RELATED_PARTY_FULL_NM
       from
          work.prim_ecm_rel_pty left join 
          work.cur_ecm_pty
             on
             (
                prim_ecm_rel_pty.RELATED_PARTY_NUMBER = cur_ecm_pty.PARTY_ID
             )
       where
          prim_ecm_rel_pty.SEGMENT_ID = cur_ecm_pty.SOURCE_SYSTEM_CD
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to perform join;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Get all current ECM PTY Rels                            * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    PARTY_X_PARTY - ecm_db.PARTY_X_PARTY                    * 
     * Target Table:    Current ECM relationships -                             * 
     *                   work.cur_ecm_rels                                      * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(ecm_db.PARTY_X_PARTY); 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete cur_ecm_rels;
    quit;

    proc sql;
       create table work.cur_ecm_rels as
          select
             PARTY_RK,
             MEMBER_PARTY_RK,
             MEMBER_TYPE_CD,
             MEMBER_DESC
       from &SOURCEDS
          where MEMBER_TYPE_CD IN ('IDENTICAL','BENE_OWNER','DIRECTOR')
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create table from ECM;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %let SOURCEDS = work.cur_ecm_rels;

    /*==========================================================================* 
     * Step:            Get all current rel info                                * 
     * Transform:       Join                                                    * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Primary (FCF/ECM) and related                           * 
     *                  (FCF/ECM) parties -                                     * 
     *                   work.prim_ecm_rel_ecm_pty                              * 
     *                  Current ECM relationships -                             * 
     *                   work.cur_ecm_rels                                      * 
     * Target Table:    All relationships - work.all_rels                       * 
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_rels;
    quit;

    proc sql;
       create table work.all_rels as
       select
          prim_ecm_rel_ecm_pty.PARTY_KEY,
          prim_ecm_rel_ecm_pty.SEGMENT_ID,
          prim_ecm_rel_ecm_pty.PARTY_NUMBER,
          prim_ecm_rel_ecm_pty.PARTY_NAME,
          prim_ecm_rel_ecm_pty.PARTY_TYPE_DESC,
          prim_ecm_rel_ecm_pty.PARTY_TAX_ID,
          prim_ecm_rel_ecm_pty.RELATIONSHIP_TYPE_CODE,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_NUMBER,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_KEY,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_TYPE_DESC,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_TAX_ID,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_NAME,
          prim_ecm_rel_ecm_pty.PARTY_RK,
          prim_ecm_rel_ecm_pty.SOURCE_SYSTEM_CD,
          prim_ecm_rel_ecm_pty.PARTY_TYPE_CD,
          prim_ecm_rel_ecm_pty.PARTY_CATEGORY_CD,
          prim_ecm_rel_ecm_pty.PARTY_SUBCATEGORY_CD,
          prim_ecm_rel_ecm_pty.PARTY_FULL_NM,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_RK,
          prim_ecm_rel_ecm_pty.RELATED_SOURCE_SYSTEM_CD,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_TYPE_CD,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_CATEGORY_CD,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_SUBCATEGORY_CD,
          prim_ecm_rel_ecm_pty.RELATED_PARTY_FULL_NM,
          cur_ecm_rels.PARTY_RK as REL_PARTY_RK,
          cur_ecm_rels.MEMBER_PARTY_RK as REL_MEMBER_PARTY_RK,
          cur_ecm_rels.MEMBER_TYPE_CD as REL_MEMBER_TYPE_CD,
          cur_ecm_rels.MEMBER_DESC as REL_MEMBER_DESC
       from
          work.prim_ecm_rel_ecm_pty left join 
          work.cur_ecm_rels
             on
             (
                prim_ecm_rel_ecm_pty.PARTY_RK = cur_ecm_rels.PARTY_RK
                and prim_ecm_rel_ecm_pty.RELATED_PARTY_RK = cur_ecm_rels.MEMBER_PARTY_RK
                and prim_ecm_rel_ecm_pty.RELATIONSHIP_TYPE_CODE = cur_ecm_rels.MEMBER_TYPE_CD
             )
       ;
    quit;
    
    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to perform join;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Split into 'new to create' and 'old to delete'          * 
     * Transform:       Splitter                                                * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    All relationships - work.all_rels                       * 
     * Target Tables:   New relationships to close -                            * 
     *                   work.new_rels_to_load                                  * 
     *                  Old relationships to be closed -                        * 
     *                   work.old_rels_to_close                                 * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.all_rels); 

    proc datasets lib=work nolist nowarn memtype = (data view);
       delete new_rels_to_load;
       delete old_rels_to_close;
    quit;

    data 
         work.new_rels_to_load 
         work.old_rels_to_close ;
       set &SOURCEDS;
       if PARTY_RK   ^=  ''  &  RELATED_PARTY_RK  ^=  '' & REL_PARTY_RK = '' & REL_MEMBER_PARTY_RK = '' then 
          output work.new_rels_to_load;
       if PARTY_RK   =  ''  &  RELATED_PARTY_RK  =  '' & REL_PARTY_RK ^= '' & REL_MEMBER_PARTY_RK ^= '' then 
          output work.old_rels_to_close;
    run;

    %fcf_rcset(&sqlrc);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create datasets from split;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Create load dataset                                     * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New relationships to close -                            * 
     *                   work.new_rels_to_load                                  * 
     * Target Table:    New CDD Party X Party creates -                         * 
     *                   iem.cdd_new_pxp                                        * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.new_rels_to_load); 

    proc datasets lib = iem nolist nowarn memtype = (data view);
      delete cdd_new_pxp;
    quit;

    data iem.cdd_new_pxp;
        if 0 then set ecm_db.party_x_party;
        set &SOURCEDS. (keep= PARTY_RK RELATED_PARTY_RK RELATIONSHIP_TYPE_CODE);
            PARTY_RK = PARTY_RK;
            MEMBER_PARTY_RK = RELATED_PARTY_RK;
            MEMBER_TYPE_CD = RELATIONSHIP_TYPE_CODE;
        drop RELATED_PARTY_RK RELATIONSHIP_TYPE_CODE;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.CDD_NEW_PXP;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    %if %fcf_get_observation_count(table=iem.cdd_new_pxp,SASDATASET=Y) lt 1 %then %goto skip_update1;
        %fcf_iem_cdd_load_rel(ds=iem.cdd_new_pxp);
    %skip_update1:

    /*==========================================================================* 
     * Step:            Create delete dataset                                   * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Old relationships to be closed -                        * 
     *                   work.old_rels_to_close                                 * 
     * Target Table:    New CDD Party X Party deletes -                         * 
     *                   iem.cdd_del_pxp                                        * 
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.old_rels_to_close); 

    proc datasets lib = iem nolist nowarn memtype = (data view);
      delete cdd_del_pxp;
    quit;

    data iem.cdd_del_pxp;
        if 0 then set ecm_db.party_x_party;
        set &SOURCEDS. (keep= PARTY_RK RELATED_PARTY_RK RELATIONSHIP_TYPE_CODE);
            PARTY_RK = PARTY_RK;
            MEMBER_PARTY_RK = RELATED_PARTY_RK;
            MEMBER_TYPE_CD = RELATIONSHIP_TYPE_CODE;
        drop RELATED_PARTY_RK RELATIONSHIP_TYPE_CODE;
    run;

    %fcf_rcset(&syserr);
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.CDD_DEL_PXP;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    %if %fcf_get_observation_count(table=iem.cdd_del_pxp,SASDATASET=Y) lt 1 %then %goto skip_update2;
        %fcf_iem_cdd_del_rel(ds=iem.cdd_del_pxp);
    %skip_update2:
    

    %MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
