/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_close_dimension_rels.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM Subject synchronization
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_close_dimension_rels(ds_HhdPty=, ds_HhdAcc=, ds_PtyAcc=)
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_close_dimension_rels(ds_HhdPty=, ds_HhdAcc=, ds_PtyAcc=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    %let obs_count_hhd_pty = %fcf_get_observation_count(table=&ds_HhdPty,SASDATASET=Y);
    %let obs_count_hhd_acc = %fcf_get_observation_count(table=&ds_HhdAcc,SASDATASET=Y);
    %let obs_count_pty_acc = %fcf_get_observation_count(table=&ds_PtyAcc,SASDATASET=Y);
    %if &obs_count_hhd_pty < 1 and &obs_count_hhd_acc < 1 and &obs_count_pty_acc < 1 %then %do;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Append                                                  *
     * Transform:       Append                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   HHD-PTY input ds - &ds_HhdPty                           * 
     *                  HHD-ACC input ds - &ds_HhdAcc                           * 
     *                  PTY-ACC input ds - &ds_PtyAcc                           * 
     * Target Table:    Append - work.all_pty_x_pty                             *
     *==========================================================================*/ 

    proc datasets lib = work nolist nowarn memtype = (data view);
       delete all_pty_x_pty;
    quit;

    data work.all_pty_x_pty;
       set &ds_HhdPty
           &ds_HhdAcc
           &ds_PtyAcc;
       keep PARTY_RK MEMBER_PARTY_RK MEMBER_TYPE_CD; 
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CLOSE_DIMENSION_RELS;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Delete                                                  *
     * Transform:       Delete                                                  * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Tables:   Append - work.all_pty_x_pty                             *
     *                  PARTY_X_PARTY - ecm_db.PARTY_X_PARTY                    *
     * Target Table:    PARTY_X_PARTY - ecm_db.PARTY_X_PARTY                    *
     *==========================================================================*/ 

    proc sql;
        delete from ecm_db.PARTY_X_PARTY as ecm
        where exists (
			select
                1
            from
                work.all_pty_x_pty as iem
            where    
                iem.PARTY_RK = ecm.PARTY_RK and
                iem.MEMBER_PARTY_RK = ecm.MEMBER_PARTY_RK and
				iem.MEMBER_TYPE_CD = ecm.MEMBER_TYPE_CD
         )
    ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CLOSE_DIMENSION_RELS;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Table Loader                                            *
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Append - work.all_pty_x_pty                             *
     * Target Table:    ECM_EVENT - ecm_db.ECM_EVENT                            *
     *==========================================================================*/ 

    %let SOURCEDS = %nrquote(work.all_pty_x_pty); 

    data _null_;
        call symput('current_dt',put(datetime(),datetime22.));
    run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_events;
    quit;

    data iem.new_events;
        if 0 then set ecm_db.ecm_event;
        set &SOURCEDS. (keep= PARTY_RK MEMBER_PARTY_RK);
        BUSINESS_OBJECT_NM = "PARTY";
        BUSINESS_OBJECT_RK = PARTY_RK;
        EVENT_TYPE_CD = "REMPAR";
        EVENT_DESC = "event.etl.load.txt";
        LINKED_OBJECT_NM = "PARTY";
        LINKED_OBJECT_RK = MEMBER_PARTY_RK;
        CREATE_USER_ID = "&ecm_user";
        CREATE_DTTM = "&current_dt"dt;
        drop PARTY_RK MEMBER_PARTY_RK;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CLOSE_DIMENSION_RELS;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    %let SOURCEDS = iem.new_events;
   
    %fcf_get_ecm_sequence_ids(in=iem.new_events,out=iem.new_events,type=event,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CLOSE_DIMENSION_RELS;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

    proc append base = ecm_db.ECM_EVENT (bulkload=&iem_bulk_load &iem_bulk_load_options)
        data = &SOURCEDS force; 
    run; 

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Error encountered executing FCF_IEM_CLOSE_DIMENSION_RELS;
        %fcf_put_error;
        %goto MACRO_END;
    %end; 

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;

