/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_alerts.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM load of FCF Alerts
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_alerts(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_alerts(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    %let obs_count = %fcf_get_observation_count(table=&ds,SASDATASET=Y);
    %if &obs_count < 1 %then %do;
        %goto MACRO_END;
    %end; 

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Incoming DS - &ds                                       *
     * Target Table:    New Alerts - work.new_alerts                            * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_alerts;
    quit;

    proc sql;
        create table work.new_alerts as
            select
                *,
                ALERT_TYPE_CD as INCIDENT_TYPE_CD,
                ALERT_CATEGORY_CD as INCIDENT_CATEGORY_CD,
                ALERT_SUBCATEGORY_CD as INCIDENT_SUBCATEGORY_CD
            from &SOURCEDS
                where LOGICAL_DELETE_IND = "N"/* & SEGMENT_ID = "&segKCSchema"*/
        ;
    quit;

    %let SOURCEDS = work.new_alerts;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create NEW_ALERTS from input table for ALT load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Alerts - work.new_alerts                            * 
     * Target Table:    New Incidents - iem.new_incidents                       * 
    *==========================================================================*/

    %let SOURCEDS = %nrquote(work.new_alerts);

    data _null_;
        call symput('current_dt',put(datetime(),datetime22.));
    run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_incidents;
    quit;

    data iem.new_incidents;
        if 0 then set ecm_db.incident_live;
        set &SOURCEDS. (keep= ALERT_ID SEGMENT_ID INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD INCIDENT_SUBCATEGORY_CD 
            OWNER_USER_LONG_ID MONEY_LAUNDERING_RISK_SCORE SCENARIO_NAME CREATE_DATE ALERT_STATUS_CODE
            ACTUAL_VALUES_TEXT SUPPRESSION_END_DATE PRIMARY_ENTITY_NAME RUN_DATE);
        VALID_FROM_DTTM = "&current_dt"dt;
        INCIDENT_ID = CATX('-','ALT',compress(put(ALERT_ID,32.)));
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        INCIDENT_DESC = PRIMARY_ENTITY_NAME;
        INCIDENT_TYPE_CD = INCIDENT_TYPE_CD;
        INCIDENT_CATEGORY_CD = INCIDENT_CATEGORY_CD;
        INCIDENT_SUBCATEGORY_CD = INCIDENT_SUBCATEGORY_CD;
        CREATE_USER_ID = "&ecm_user";
        CREATE_DTTM =  "&current_dt"dt;
        VERSION_NO = 1;
        DELETE_FLG = "0";
        INCIDENT_DISPOSITION_CD = ALERT_STATUS_CODE;
        INVESTIGATOR_USER_ID = OWNER_USER_LONG_ID;
        X_SASFCS_ML_RS = MONEY_LAUNDERING_RISK_SCORE;
        X_SASFCS_RSK_CLSSF = .;
        X_SASFCS_PRP_RSK_CLSSF = .;
        X_SASFCS_SCEN_NM = SCENARIO_NAME;
        X_SASFCS_CREATE_DT = CREATE_DATE;
        X_ALT_RUN_DATE = RUN_DATE;
        X_SASFCS_ACT_VAL_TXT = ACTUAL_VALUES_TEXT;
        X_SUPPRESSION_END_DATE = SUPPRESSION_END_DATE;
        drop ALERT_ID SEGMENT_ID OWNER_USER_LONG_ID MONEY_LAUNDERING_RISK_SCORE SCENARIO_NAME
            ALERT_STATUS_CODE CREATE_DATE ACTUAL_VALUES_TEXT SUPPRESSION_END_DATE RUN_DATE PRIMARY_ENTITY_NAME;
    run;

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for ALT load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_get_ecm_sequence_ids(in=iem.new_incidents,out=iem.new_incidents,type=incident,block=&iem_performance_load);

    %fcf_rcset(&syserr);
    
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM INCIDENT primary keys for ALT load;
        %fcf_put_error;
        %goto MACRO_END;
    %end;    

    %fcf_iem_load_incident(type='alert',ds=iem.new_incidents);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
