/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_load_risks.sas
| PRODUCT: SAS Financial Crimes Foundation
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM load of FCF Risk Assessments
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_load_risks(ds=);
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_load_risks(ds=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    %let obs_count = %fcf_get_observation_count(table=&ds,SASDATASET=Y);
    %if &obs_count < 1 %then %do;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Extract                                                 * 
     * Transform:       Extract                                                 * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    Incoming DS - &ds                                       * 
     * Target Table:    New Risk Assessment -                                   * 
     *                   work.new_risk_assessments                              * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(&ds);

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete new_risk_assessments;
    quit;

    proc sql;
        create table work.new_risk_assessments as
            select
                *,
                ASSESSMENT_TYPE_CD as INCIDENT_TYPE_CD,
                ASSESSMENT_CATEGORY_CD as INCIDENT_CATEGORY_CD,
                ASSESSMENT_SUBCATEGORY_CD as INCIDENT_SUBCATEGORY_CD
            from &SOURCEDS
                where LOGICAL_DELETE_IND = "N"/* & SEGMENT_ID = "&segKCSchema"*/
        ;
    quit;

    %let SOURCEDS = work.new_risk_assessments;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create NEW_RISK_ASSESSMENTS for RAS load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    /*==========================================================================* 
     * Step:            Table Loader                                            * 
     * Transform:       Table Loader (Version 2.1)                              * 
     * Description:                                                             * 
     *                                                                          * 
     * Source Table:    New Risk Assessment -                                   * 
     *                   work.new_risk_assessments                              * 
     * Target Table:    New Incidents - iem.new_incidents                       * 
     *==========================================================================*/

    %let SOURCEDS = %nrquote(work.new_risk_assessments);

    data _null_;
        call symput('current_dt',put(datetime(),datetime22.));
    run;

    proc datasets lib = iem nolist nowarn memtype = (data view);
        delete new_incidents;
    quit;

    data iem.new_incidents;
        if 0 then set ecm_db.incident_live;
        set &SOURCEDS. (keep= RISK_ASSESSMENT_ID SEGMENT_ID INCIDENT_TYPE_CD INCIDENT_CATEGORY_CD INCIDENT_SUBCATEGORY_CD 
            RISK_CLASSIFICATION PROPOSED_RISK_CLASSIFICATION RISK_ASSESSMENT_STATUS_CODE OWNER_USER_LONG_ID CREATE_DATE);
        VALID_FROM_DTTM = "&current_dt"dt;
        INCIDENT_ID = CATX('-','RAS',compress(put(RISK_ASSESSMENT_ID,32.)));
        SOURCE_SYSTEM_CD = SEGMENT_ID;
        INCIDENT_TYPE_CD = INCIDENT_TYPE_CD;
        INCIDENT_CATEGORY_CD = INCIDENT_CATEGORY_CD;
        INCIDENT_SUBCATEGORY_CD = INCIDENT_SUBCATEGORY_CD;
        CREATE_USER_ID = "&ecm_user";
        CREATE_DTTM =  "&current_dt"dt;
        UPDATE_USER_ID = "&ecm_user";
        VERSION_NO = 1;
        DELETE_FLG = "0";
        INCIDENT_DISPOSITION_CD = RISK_ASSESSMENT_STATUS_CODE;
        INVESTIGATOR_USER_ID = OWNER_USER_LONG_ID;
        X_SASFCS_ML_RS = .;
        X_SASFCS_RSK_CLSSF = RISK_CLASSIFICATION;
        X_SASFCS_PRP_RSK_CLSSF = PROPOSED_RISK_CLASSIFICATION;
        X_SASFCS_SCEN_NM = "";
        X_SASFCS_ACT_VAL_TXT = "";
        X_SUPPRESSION_END_DATE = .;
        X_ALT_RUN_DATE =.;
        X_SASFCS_CREATE_DT = CREATE_DATE;
        drop RISK_ASSESSMENT_ID SEGMENT_ID RISK_CLASSIFICATION PROPOSED_RISK_CLASSIFICATION RISK_ASSESSMENT_STATUS_CODE
            OWNER_USER_LONG_ID CREATE_DATE ;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create IEM.NEW_INCIDENTS for RAS load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete temp_map;
    quit;

    %fcf_get_ecm_sequence_ids(in=iem.new_incidents,out=iem.new_incidents,type=incident,block=&iem_performance_load);

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to generate ECM INCIDENT primary keys for RAS load;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    %fcf_iem_load_incident(type='risk-assessment',ds=iem.new_incidents);

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else
        %do;
            %fcf_execute_macro(fcf_success);
        %end;
%mend;
