/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_iem_stage_alert.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    FCF to ECM load of FCF Alerts
+------------------------------------------------------------------------------+
| USAGE:   %fcf_iem_stage_alert(StagedNew=)
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_iem_stage_alert(StagedNew=);

    /*------------------------------------------
     * Set error code to 0 and message to blank
     *------------------------------------------*/

    %let trans_rc = 0;
    %let _fcfmsg_=;

    data _null_; 

        x=index("&StagedNew",'.');
        if x gt 0 then do;
            call symput( 'StagedNewLib' , substr("&StagedNew", 1, index("&StagedNew", '.') - 1)); 
            call symput ( 'StagedNewDs' , trim(left(substr("&StagedNew", index("&StagedNew", '.') + 1))));
        end;
        else do;
            call symput( 'StagedNewLib' , 'work');
            call symput( 'StagedNewDs' , "&StagedNew");
        end;
    run;

    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to split input parameter for ALT staging;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete ecm_alt;
    quit;

    proc sql;
        create table work.ecm_alt as 
            select
                incident_id
            from
                ecm_db.incident_live
            where
                incident_type_cd = 'AMLALT'
            order by
                incident_id
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create ECM_ALT for ALT staging;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

    proc datasets lib = work nolist nowarn memtype = (data view);
        delete fcf_alt;
    quit;

    proc sql;
        create table work.fcf_alt as
            select
                *,
                catx('-','ALT',alert_id) as incident_id
            from
                &db_kc_libname..FSK_ALERT
            where
                (ALERT_STATUS_CODE = 'ACT' or ALERT_STATUS_CODE = 'SUE') and
				(ALERT_TYPE_CD <> 'MANUAL' and ALERT_SUBCATEGORY_CD <> 'MANUAL')
            order by
                incident_id
        ;
    quit;

    %fcf_rcset(&sqlrc);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create FCF_ALT for ALT staging;

        %fcf_put_error;
        %goto MACRO_END;
    %end;
                
    proc datasets lib = &StagedNewLib nolist nowarn memtype = (data view);
        delete &StagedNewDs;
    quit;

    data &StagedNew (drop = incident_id);                                   
        merge work.fcf_alt (in=inFCF) work.ecm_alt (in=inECM);
        by incident_id;
        if inFCF and not inECM then do;
            output &StagedNew;
        end;   
    run;

    
    %fcf_rcset(&syserr);

    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
        %let _fcfmsg_=ERROR: FCF: IEM: Unable to create &StagedNew for ALT staging;

        %fcf_put_error;
        %goto MACRO_END;
    %end;

%MACRO_END:

    %if &trans_rc eq 3 or &trans_rc ge 5 %then
        %fcf_execute_macro(fcf_error);
    %else %do;
        %fcf_execute_macro(fcf_success);
    %end;

%mend;
