/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    fcf_sa_deploy_scenarios.sas
| PRODUCT: SAS Compliance Solutions
| SYSTEM:  Windows, Unix
| DESC:    Update Scenario Administrator tables by extracting
|          the data from ECM and download scenarios, risk factors and headers
|          source code from the SAS Content server.
+------------------------------------------------------------------------------+
| PARAMETERS:
|          scr_dav_path - Content server path for scenario sources, for example,
|                         Products/SASComplianceSolutions/scenarios/
|          hdr_dav_path - Content server path for header sources, for example,
|                         Products/SASComplianceSolutions/headers/
|          fcf_debug_flg - if Y save all work datasets in !FCFROOT/custom
|                          and enable full SAS log
+------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2002-2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro fcf_sa_deploy_scenarios(scr_dav_path=, hdr_dav_path=, fcf_debug_flg=N);

    %macro fcf_debug_setup();
        %if &fcf_debug_flg eq Y %then %do;
            options spool mprint mlogic symbolgen mprintnest mlogicnest;
            libname dbg "!FCFROOT/custom";
        %end;
    %mend fcf_debug_setup;
    %fcf_debug_setup();

    %macro fcf_debug_copy();
        %if &fcf_debug_flg eq Y %then %do;
            proc copy inlib=work outlib=dbg;
            run;
        %end;
    %mend fcf_debug_copy;

    %macro fcf_sa_abort(msg=, p1=, p2=, p3=, p4=);
         %let parm_list=;
         %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
         %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
         %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
         %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

         %let _fcfmsg_=ERROR: %sysfunc(sasmsg(sashelp.fcfmsg, &msg, N &parm_list));
         %let trans_rc = 99;
         %fcf_put_error
         %fcf_clear_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds)
         %ABORT CANCEL &trans_rc;
    %mend fcf_sa_abort;

    %macro fcf_sa_abort_on_error(msg=, p1=, p2=, p3=, p4=);
     %fcf_chkrc
     %if &bat_abort=Y %then %do;
         %let sys_err_msg = &bat_errmsg;
         %let parm_list=;
         %if "&p1" ne "" %then %do;  %let parm_list =, &p1;             %end;
         %if "&p2" ne "" %then %do;  %let parm_list =&parm_list , &p2;  %end;
         %if "&p3" ne "" %then %do;  %let parm_list =&parm_list , &p3;  %end;
         %if "&p4" ne "" %then %do;  %let parm_list =&parm_list , &p4;  %end;

         %let _fcfmsg_=ERROR: Update failed: %sysfunc(sasmsg(sashelp.fcfmsg, &msg, N &parm_list));
         %let trans_rc = 99;
         %fcf_put_error
         %fcf_clear_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds)
         %ABORT CANCEL &trans_rc;
     %end;
    %mend fcf_sa_abort_on_error;

    %macro fcf_sa_create_fsk_backups;
        libname fcfsabk  "&fcf_sa_backup_folder";

        data fcfsabk.fsk_header (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_header;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_header, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_header_element (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_header_element;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_header_element, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_scenario (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_scenario;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_scenario, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_scenario_parameter (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_scenario_parameter;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_scenario_parameter, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_scenario_alert_column (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_scenario_alert_column;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_scenario_alert_column, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_routing_group (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_routing_group;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_routing_group, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_routing_rule (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_routing_rule;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_routing_rule, p2=&fcf_sa_backup_folder)
        data fcfsabk.fsk_routing_constraint (GENMAX=&fcf_sa_genmax);
             set seg_kc.fsk_routing_constraint;
        run;
        %fcf_sa_abort_on_error(msg=sa_cannot_backup_ds, p1=fsk_routing_constraint, p2=&fcf_sa_backup_folder)
    %mend fcf_sa_create_fsk_backups;

    /* lock on the dataset to make sure that only one critical job is running */
    %macro fcf_sa_sync_lock();
        %fcf_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds)
        %if &fcf_sync_lock_failed eq Y %then %do;
             %fcf_sa_abort(msg=fcf_critical_update_in_process, p1=&fcf_sync_job_ds)
        %end;
    %mend fcf_sa_sync_lock;

    %fcf_sa_sync_lock();

    /* create data set to store errors */
    data errors;
        length msg $200;
        stop;
    run;

    /* some of the code above may set SYSRC to a negative value, so reset */
    %let sysrc = 0;

    /****************************************************************************
     ****************************************************************************
     **         Backup existing tables before updating them
     ****************************************************************************
     ****************************************************************************/
    %fcf_sa_create_fsk_backups

    /****************************************************************************
     ****************************************************************************
     **          Create data for FSK_HEADER table
     ****************************************************************************
     ****************************************************************************/

    /*********          HEADERS    *****************/
    /* select all CHAR UDF columns for all headers */
    proc sql;
        create table headersc as
        select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE
                from ecm_db.party_live p
                inner join ecm_db.party_udf_char_value c
                on c.udf_table_nm = "PARTY"
                   and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                where party_category_cd = "HEADER"
                order by party_rk
        ;
    quit;
    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=headersc)

    proc transpose data=headersc out=hdrc;
         id UDF_NM;
         var UDF_VALUE;
         by PARTY_RK;
    run;

    /* select all DATE UDF columns for all headers */
    proc sql;
        create table headersd as
        select  p.party_rk, d.udf_nm, d.udf_value
                from ecm_db.party_live p
                inner join ecm_db.party_udf_date_value d
                on d.udf_table_nm = "PARTY"
                   and d.valid_from_dttm = p.valid_from_dttm and p.party_rk = d.party_rk
                where party_category_cd = "HEADER"
                order by party_rk
        ;
    quit;
    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=headersd)

    /* pivot the table */
    proc transpose data=headersd out=hdrdt;
         id UDF_NM;
         var UDF_VALUE;
         by PARTY_RK;
    run;

    /* drop the columns that are noit part of a header */
    data hdrd;
        set hdrdt;
        drop _LABEL_  _NAME_;
    run;

    /* select all NUMERIC UDF columns for all headers */
    proc sql;
        create table headersn as
        select  p.party_rk, n.udf_nm, n.udf_value
                from ecm_db.party_live p
                inner join ecm_db.party_udf_num_value n
                on n.udf_table_nm = "PARTY"
                   and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                where party_category_cd = "HEADER"
                order by party_rk
        ;
    quit;
    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=headersn)

    /* pivot the table */
    proc transpose data=headersn out=hdrnt;
         id UDF_NM;
         var UDF_VALUE;
         by PARTY_RK;
    run;

    data hdrn;
        set hdrnt;
        drop _LABEL_  _NAME_;
    run;

    /* join all UDF columns in one table */
    proc sql;
        create table hdrjoin
            as select c.*, d.*, n.*, p.VERSION_NO
            from hdrc c
            left join hdrd d
                on c.party_rk = d.party_rk
            left join hdrn n
                on c.party_rk = n.party_rk
            inner join ecm_db.party_live p
                on c.party_rk = p.party_rk
        ;
    quit;
    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=hdrjoin)

    %macro fcf_sa_error_if_empty(tab_obs=, err_msg=);
        %if &tab_obs eq 0 %then %do;
            %fcf_sa_abort(msg=&err_msg);
        %end;
    %mend fcf_sa_error_if_empty;
    %fcf_sa_error_if_empty(tab_obs=&SQLOBS, err_msg=sa_no_headers);

    /* add columns from PARTY table, HDRALL is used later for scenarios to create relationships in FCF tables */
    proc sql;
        create table HDRALL as
        select  h.*,
                p.valid_from_dttm as valid_from_dttm,
                p.update_user_id as LSTUPDATE_USER_ID,
                p.CREATE_USER_ID
            from hdrjoin h
            inner join ecm_db.party_live p
            on h.party_rk = p.party_rk
        where h.logical_delete_ind ne "Y"
        ;
    quit;

    proc sql;
        create table HDRDEL as
        select  input(h.header_id, BEST32. ) as header_id,
                p.valid_from_dttm as valid_from_dttm,
                p.update_user_id as LSTUPDATE_USER_ID,
                p.CREATE_USER_ID
            from HDRJOIN h
            inner join ecm_db.party_live p
            on h.party_rk = p.party_rk
        where h.logical_delete_ind eq "Y"
        ;
    quit;

    /* fix the datatypes to match FCF tables*/
    data hdrnondel;
        length HEADER_LOCATION_DESC $255; /* need this in case all headers do not have source */
        format LSTUPDATE_DATE datetime20.;
        set HDRALL (rename=(header_id=x_header_id VERSION_NO=VERSION_NUMBER));
        drop _LABEL_  _NAME_ x_header_id X_CREATE_USER_ID  VALID_FROM_DTTM;
        length header_id 8.;
        length DAV_FILE_NAME $255;
            LSTUPDATE_DATE = input(put (VALID_FROM_DTTM, datetime20.), datetime20.);
            header_id = input(x_header_id, BEST32. );
            if CREATE_DATE = . then CREATE_DATE = DATETIME();
            /* double checking, name should be empty anyway, uidef sets it to empty using the same condition */
            if (SOURCE_TYPE_DESC eq 'SAS Datastep' or SOURCE_TYPE_DESC ne 'HP Datastep') and HEADER_AUTOGEN_IND ne 'Y' or SOURCE_TYPE_DESC ne 'Custom'
            then do;
                DAV_FILE_NAME = HEADER_LOCATION_DESC;
                HEADER_LOCATION_DESC = "&fcf_sa_hdr_path" || left(trim(HEADER_NAME)) ||  '_v_' || trim(left(VERSION_NUMBER)) || ".sas";
            end;
            else do;
                DAV_FILE_NAME = " ";
                HEADER_LOCATION_DESC = ' ';
            end;
    run;
    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=hdrnondel)

    /* fsk_header table cannot be wiped out because of the database constraints,
     * it has to be updated in place. So we create two groups of records,
     * existing headers that will be updated and new headers that will be inserted.
     * Select records that exist in fsk_header and there is newer version exists in
     * hdrnondel - these records have to be updated,
     * records that are not changed will not be updated */
    proc sql;
        create table HDRUPD
            as select * from hdrnondel f
            where exists(select header_id from seg_kc.fsk_header h
                                where f.header_id eq h.header_id
                                      and (f.LSTUPDATE_DATE ne h.LSTUPDATE_DATE
                                           or h.LSTUPDATE_DATE IS NULL
                                           or h.logical_delete_ind="Y")
                        )
        order by header_id;
        ;
    quit;
    proc sql;
        create table HDRUNDELETED as
            select * from hdrnondel f
            where exists(select header_id from seg_kc.fsk_header h
                                where f.header_id eq h.header_id
                                      and h.logical_delete_ind="Y"
                        )
        order by header_id;
        ;
    quit;

    /* select records that have to be added to update fsk_header */
    proc sql;
        create table HDRNEW
            as select * from hdrnondel
            where header_id not in (select header_id from seg_kc.fsk_header)
        ;
    quit;

    /* make the dataset of the same type as fsk_header so that insert will have the same order of columns */
    data HDRINS;
        set seg_kc.fsk_header;
        stop;
    run;

    proc append base=HDRINS data=HDRNEW FORCE NOWARN;
    run;

    /****************************************************************************
     ****************************************************************************
     **          Create data for FSK_HEADER_ELEMENTS table
     ****************************************************************************
     ****************************************************************************/
    %let elem_cnt=0;
    %macro fcf_sa_get_fsk_header();
        /* update FSK_HEADER_ELEMENTS */
        /* make the dataset of the same type as fsk_header_element so that insert will have the same order of columns */
         data colins;
             set seg_kc.fsk_header_element;
             stop;
         run;

        /* select all CHAR UDF columns for all columns */
        proc sql;
            create table columnsc as
            select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE, c.row_no
                    from ecm_db.party_live p
                    inner join ecm_db.party_udf_char_value c
                    on c.udf_table_nm = "X_DS_COL"
                       and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                    where p.party_category_cd = "HEADER"
                    order by party_rk, row_no
            ;
        quit;
        %let elem_cnt=&SQLOBS;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=columnsc)

        %if &elem_cnt ne 0 %then %do;
            proc transpose data=columnsc out=colc;
                 id UDF_NM;
                 var UDF_VALUE;
                 by PARTY_RK ROW_NO;
            run;

            /* select all NUMERIC UDF columns for all columns */
            proc sql;
                create table columnsn as
                select  p.party_rk, n.udf_nm, n.udf_value, n.row_no
                        from ecm_db.party_live p
                        inner join ecm_db.party_udf_num_value n
                        on n.udf_table_nm = "X_DS_COL"
                           and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                        where p.party_category_cd = "HEADER"
                        order by party_rk, row_no
                ;
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=columnsn)

            /* pivot the table */
            proc transpose data=columnsn out=colnt;
                 id UDF_NM;
                 var UDF_VALUE;
                 by PARTY_RK ROW_NO;
            run;

            data coln;
                set colnt;
                drop _LABEL_  _NAME_;
            run;

            /* ECM does not store full fsk_header_element */
            /* need to add ELEMENT_TYPE_CODE ELEMENT_LENGTH_COUNT ELEMENT_FORMAT_CODE ELEMENT_LABEL_DESC */
            /* create full element info */
            proc sql noprint;
                     create table dataset_list as
                     select UPCASE(memname) as DATASET_NAME
                     from dictionary.tables
                     where libname="&fcf_sa_prep_lib"
                     order by DATASET_NAME
                     ;
            quit;

            data elements;
               set dataset_list;
               length  ELEMENT_TYPE_CODE   $6
                    ELEMENT_LENGTH_COUNT    8
                    ELEMENT_FORMAT_CODE    $8
                    ELEMENT_LABEL_DESC    $35
                    ELEMENT_NAME          $35 ;
               keep  dataset_name ELEMENT_TYPE_CODE ELEMENT_LENGTH_COUNT ELEMENT_FORMAT_CODE ELEMENT_LABEL_DESC ELEMENT_NAME;
               dsid=open("&fcf_sa_prep_lib.." || DATASET_NAME, "i");
               num=attrn(dsid,"nvars");
               do i=1 to num;
                  element_name=upcase(varname(dsid,i));
                  ELEMENT_TYPE_CODE=vartype(dsid,i);
                  ELEMENT_FORMAT_CODE=varfmt(dsid,i);
                  ELEMENT_LABEL_DESC=varlabel(dsid,i);
                  ELEMENT_LENGTH_COUNT=varlen(dsid,i);
                  if  ELEMENT_LABEL_DESC = ' ' then ELEMENT_LABEL_DESC = element_name;
                  output;
               end;
               rc=close(dsid);
            run;

            /* join all UDF columns in one table, exclude elements for deleted headers and exclude not found data sets */
            proc sql;
                create table colall
                    as select c.*, n.*, e.ELEMENT_TYPE_CODE, e.ELEMENT_LENGTH_COUNT, e.ELEMENT_FORMAT_CODE, e.ELEMENT_LABEL_DESC
                    from colc c
                    left join coln n
                        on c.party_rk = n.party_rk AND c.row_no = n.row_no
                    inner join hdrall h
                        on h.header_id = c.header_id and c.party_rk = h.party_rk
                    inner join elements e
                        on h.denorm_table_name = e.dataset_name and e.element_name = c.element_name
                ;
            quit;

            /* find missing datasets */
            proc sql;
                create table ds_errors as
                select h.denorm_table_name
                from hdrall h
                where h.denorm_table_name not in (select dataset_name from elements)
                      and h.denorm_table_name ne "" and h.HEADER_AUTOGEN_IND eq "Y" and h.SOURCE_TYPE_DESC eq 'SAS Datastep'
            ;
            quit;

            %let ds_errors_cnt = &SQLOBS;

            /* changing the datatypes to match FCF tables*/
            data colfull;
                set colall (rename=(header_id=x_header_id));
                drop _LABEL_ ROW_NO _NAME_ PARTY_RK x_header_id;
                length header_id 8.;
                header_id = input(x_header_id, BEST32. );
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=colfull)

            proc append base=colins data=colfull FORCE NOWARN;
            run;
        %end;
    %mend fcf_sa_get_fsk_header;

    %fcf_sa_get_fsk_header();


    /****************************************************************************
     ****************************************************************************
     **          Process ROUTING GROUPS
     ****************************************************************************
     ****************************************************************************/
    /* routing groups are processed before scenarios are updated, so that references could be
     * updated (fsk_scenario.routing_group_id) */

    %let rg_empty = 1; /* set to 1 when there are no routing groups */
    %macro fcf_sa_update_rg();
        /* select all CHAR UDF columns for all RG      */
        proc sql;
            create table rgroupsc as
            select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE
                    from ecm_db.party_live p
                    inner join ecm_db.party_udf_char_value c
                    on c.udf_table_nm = "PARTY"
                       and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                    where party_category_cd = "ROUTINGGRP"
                    order by party_rk
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rgroupsc)

        %if &SQLOBS ne 0 %then %do;
            %let rg_empty = 0;
            proc transpose data=rgroupsc out=rgall1;
                 id UDF_NM;
                 var UDF_VALUE;
                 by PARTY_RK;
            run;
            /* exclude deleted groups. rgall is used later to filter RR and RC*/
            proc sql;
                create table rgall as
                select * from rgall1
                where logical_delete_ind ne "Y"
                ;
            quit;
            /* fix the datatypes to match FCF tables*/
            data rgfull;
                set rgall (rename=(routing_group_id=x_routing_group_id));
                drop _LABEL_  _NAME_ PARTY_RK x_routing_group_id;
                length routing_group_id 8.;
                routing_group_id = input(x_routing_group_id, BEST32. );
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rgfull)

            /* make the dataset of the same type as fsk_rgroup so that insert will have the same order of columns */
            data rgins;
                set seg_kc.fsk_routing_group;
                stop;
            run;

            /* select records that exist in fsk_routing_group - they will be updated */
            proc sql;
                create table rgupd
                    as select * from rgfull
                    where routing_group_id in (select routing_group_id from seg_kc.fsk_routing_group)
                ;
            quit;
            %let rgCnt = &SQLOBS;

            /* select records that have to be added to update fsk_routing_group */
            proc sql;
                create table rgnew
                    as select * from rgfull
                    where routing_group_id not in (select routing_group_id from seg_kc.fsk_routing_group)
                ;
            quit;

            proc append base=rgins data=rgnew FORCE NOWARN;
            run;

            /* update FSK_ROUTING_RULE */
            /* select all CHAR UDF columns for all RR */
            proc sql;
                create table rulesc as
                select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE, c.row_no
                        from ecm_db.party_live p
                        inner join ecm_db.party_udf_char_value c
                        on c.udf_table_nm = "X_RR"
                           and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                        where p.party_category_cd = "ROUTINGGRP"
                        order by party_rk, row_no
                ;
            quit;
            %let char_rec = &SQLOBS;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rulesc)

            proc transpose data=rulesc out=rrc;
                 id UDF_NM;
                 var UDF_VALUE;
                 by PARTY_RK ROW_NO;
            run;

            /* select all NUMERIC UDF rules for all rules */
            proc sql;
                create table rulesn as
                select  p.party_rk, n.udf_nm, n.udf_value, n.row_no
                        from ecm_db.party_live p
                        inner join ecm_db.party_udf_num_value n
                        on n.udf_table_nm = "X_RR"
                           and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                        where p.party_category_cd = "ROUTINGGRP"
                        order by party_rk, row_no
                ;
            quit;
            %let num_rec = &SQLOBS;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rulesn)

            %if &num_rec ne 0 %then %do;
                /* pivot the table */
                proc transpose data=rulesn out=rrnt;
                     id UDF_NM;
                     var UDF_VALUE;
                     by PARTY_RK ROW_NO;
                run;

                data rrn;
                    set rrnt;
                    drop _LABEL_  _NAME_;
                run;
            %end;

            /* make the dataset of the same type as fsk_header_element so that insert will have the same order of rules */
            data rrins;
                set seg_kc.fsk_routing_rule;
                stop;
            run;

            /* each Rule record must have both numeric and char fields - because they are marked as required in uidef */
            %let rr_count = 0;
            %if &char_rec ne 0 and &num_rec ne 0 %then %do;
                %let rr_count = &char_rec;
                /* join all UDF rules in one table, exclude rule for deleted groups */
                proc sql;
                    create table rrall
                        as select c.*, n.*
                        from rrc c
                        left join rrn n
                            on c.party_rk = n.party_rk
                               AND c.row_no = n.row_no
                        inner join rgall g
                            on g.routing_group_id = c.routing_group_id
                    ;
                quit;

                /* fix the datatypes to match FCF tables*/
                data rrfull;
                    set rrall (rename=(ROUTE_ORDER_NUMBER=x_ROUTE_ORDER_NUMBER routing_group_id=x_routing_group_id routing_rule_id=x_routing_rule_id));
                    drop _LABEL_ ROW_NO _NAME_ PARTY_RK x_ROUTE_ORDER_NUMBER x_routing_group_id x_routing_rule_id;
                    length ROUTE_ORDER_NUMBER 8.;
                    ROUTE_ORDER_NUMBER = input(x_ROUTE_ORDER_NUMBER, BEST32. );
                    length routing_group_id 8.;
                    routing_group_id = input(x_routing_group_id, BEST32. );
                    length routing_rule_id 8.;
                    routing_rule_id = input(x_routing_rule_id, BEST32. );
                run;
                %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rrfull)

                proc append base=rrins data=rrfull FORCE NOWARN;
                run;
                %fcf_debug_copy();
                proc datasets lib=work nolist;
                   delete rrfull;
                quit;
            %end;

            /* update FSK_ROUTING_CONSTRAINT */
            /* select all CHAR UDF columns for all rc */
            proc sql;
                create table constraintsc as
                select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE, c.row_no
                        from ecm_db.party_live p
                        inner join ecm_db.party_udf_char_value c
                        on c.udf_table_nm = "X_RC"
                           and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                        where p.party_category_cd = "ROUTINGGRP"
                        order by party_rk, row_no
                ;
            quit;
            %let rc_char_rec = &SQLOBS;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=constraintsc)

            %if &rc_char_rec ne 0 %then %do;
                proc transpose data=constraintsc out=rcc;
                     id UDF_NM;
                     var UDF_VALUE;
                     by PARTY_RK ROW_NO;
                run;
            %end;

            /* select all NUMERIC UDF constraints for all constraints */
            proc sql;
                create table constraintsn as
                select  p.party_rk, n.udf_nm, n.udf_value, n.row_no
                        from ecm_db.party_live p
                        inner join ecm_db.party_udf_num_value n
                        on n.udf_table_nm = "X_RC"
                           and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                        where p.party_category_cd = "ROUTINGGRP"
                        order by party_rk, row_no
                ;
            quit;
            %let rc_num_rec = &SQLOBS;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=constraintsn)

            /* make the dataset of the same type as fsk_routing_constraint so that insert will have the same order of columns */
            data rcins;
                set seg_kc.fsk_routing_constraint;
                stop;
            run;

            %if &rc_num_rec ne 0 %then %do;
                /* pivot the table */
                proc transpose data=constraintsn out=rcnt;
                     id UDF_NM;
                     var UDF_VALUE;
                     by PARTY_RK ROW_NO;
                run;

                data rcn;
                    set rcnt;
                    drop _LABEL_  _NAME_;
                run;


                /* join all UDF constraints in one table, exclude constraints for deleted rules (and thus for deleted groups) */
                proc sql;
                    create table rcall
                        as select c.*, n.*
                        from rcc c
                        left join rcn n
                            on c.party_rk = n.party_rk
                               AND c.row_no = n.row_no
                        inner join rrall r
                            on r.routing_rule_id = c.routing_rule_id and r.routing_group_id = c.routing_group_id
                    ;
                quit;
           %end;

           /* each constraint must have both numeric and char fields - because they are marked as required in uidef */
           %let rc_count = 0;
           %if &rc_char_rec ne 0 and &rc_num_rec ne 0 %then %do;
                %let rc_count = &rc_char_rec;
                /* fix the datatypes to match FCF tables*/
                data rcfix;
                    set rcall (rename=(CONSTRAINT_ORDER_NUMBER=x_CONSTRAINT_ORDER_NUMBER routing_group_id=x_routing_group_id routing_rule_id=x_routing_rule_id));
                    drop _LABEL_ ROW_NO _NAME_ PARTY_RK x_CONSTRAINT_ORDER_NUMBER x_routing_group_id x_routing_rule_id;
                    length CONSTRAINT_ORDER_NUMBER 8.;
                    CONSTRAINT_ORDER_NUMBER = input(x_CONSTRAINT_ORDER_NUMBER, BEST32. );
                    length routing_group_id 8.;
                    routing_group_id = input(x_routing_group_id, BEST32. );
                    length routing_rule_id 8.;
                    routing_rule_id = input(x_routing_rule_id, BEST32. );
                run;
                %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rcfix)

                /* filter routing constrains using routing_group_id - workaround for ECM design flaw */
                proc sql;
                    create table rcfull as
                        select * from rcfix
                        where routing_rule_id in (select routing_rule_id from rrins)
                    ;
                quit;
                %fcf_sa_abort_on_error(msg=sa_processing_error, p1=rcfull)

                proc append base=rcins
                            data=rcfull FORCE NOWARN;
                run;
                %fcf_debug_copy();
                proc datasets lib=work nolist;
                   delete rcfull;
                quit;
           %end;

            /* update all routing tables */
            proc sql;
                delete from seg_kc.fsk_routing_constraint;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_constraint)
            proc sql;
                delete from seg_kc.fsk_routing_rule;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_rule)

            /* update existing records in fsk_routing_group - cannot delete because of fsk_sceanario foreign key constraint */
            /* mark all records as deleted */
            proc sql;
                    update seg_kc.fsk_routing_group
                        set LOGICAL_DELETE_IND="Y"
                        where routing_group_id ne -1
                    ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_group)

            /* delete records that are not referenced any  more */
            proc sql;
                delete from seg_kc.fsk_routing_group
                where routing_group_id not in (select routing_group_id from seg_kc.fsk_scenario)
                      and routing_group_id not in (select routing_group_id from rrins)
                      and routing_group_id ne -1
                      and routing_group_id not in (select routing_group_id from rgupd)
                ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_group)
            %if &rgCnt ne 0 %then %do; /* modify fails if rgupd is empty */
                data seg_kc.fsk_routing_group;
                     modify seg_kc.fsk_routing_group
                            rgupd
                            UPDATEMODE=NOMISSINGCHECK;
                     BY routing_group_id;
                     if _iorc_= 0 then replace;
                run;
                %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_group)
            %end;

            /* add new routing groups */
            proc sql;
                insert into seg_kc.fsk_routing_group
                    select * from rgins
                ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_group)
            %if &rr_count ne 0 %then %do;
                proc sql;
                    insert into seg_kc.fsk_routing_rule
                      select * from rrins
                    ;
                quit;
                %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_rule)
                %if &rr_count ne 0 and &rc_count ne 0 %then %do;
                    proc sql;
                        insert into seg_kc.fsk_routing_constraint
                          select * from rcins
                        ;
                    quit;
                    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_constraint)
                %end;
            %end;
            %fcf_debug_copy();
            proc datasets lib=work nolist;
               delete rcall rcc rcins rcn rcnt rgins rgnew rgroupsc rgupd rrall rrc rrins rrn rrnt rulesc rulesn rgfull;
            quit;
        %end;
        %else %do;
            /* no routing groups - so delete all groups, rules and constraints */
            proc sql;
                delete from seg_kc.fsk_routing_constraint;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_constraint)
            proc sql;
                delete from seg_kc.fsk_routing_rule;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_rule)
            /* mark all records as deleted */
            proc sql;
                    update seg_kc.fsk_routing_group
                        set LOGICAL_DELETE_IND="Y"
                        where routing_group_id ne -1;
                    ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_group)
            /* delete records that are not referenced */
            proc sql;
                delete from seg_kc.fsk_routing_group
                where routing_group_id not in (select routing_group_id from seg_kc.fsk_scenario)
                and routing_group_id ne -1;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_routing_group)
            %fcf_debug_copy();
            proc datasets lib=work nolist;
               delete rgroupsc;
            quit;
        %end;
    %mend fcf_sa_update_rg;

    /* update routing groups */
    %fcf_sa_update_rg()

    /****************************************************************************
     ****************************************************************************
     **          Create data for fsk_scenario table
     ****************************************************************************
     ****************************************************************************/
    %macro fcf_sa_extract_scenarios();
        /*********          SCENARIOS   *****************/
        /* select all CHAR UDF columns for all scenarios */
        proc sql;
            create table scenariosc as
            select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE
                    from ecm_db.party_live p
                    inner join ecm_db.party_udf_char_value c
                    on c.udf_table_nm = "PARTY"
                       and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                    where party_category_cd = "SCENARIO"
                    order by party_rk
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=scenariosc)

        proc transpose data=scenariosc out=scrct;
             id UDF_NM;
             var UDF_VALUE;
             by PARTY_RK;

        run;

        /* drop the columns that are not part of a scenario. Drop header_id and routing_group_id that will be
         * populated based on the relationships */
        data scrc;
            set scrct;
            drop _LABEL_  _NAME_ header_id routing_group_id;
        run;

        /* select all DATE UDF columns for all scenarios */
        proc sql;
            create table scenariosd as
            select  p.party_rk, d.udf_nm, d.udf_value
                    from ecm_db.party_live p
                    inner join ecm_db.party_udf_date_value d
                    on d.udf_table_nm = "PARTY"
                       and d.valid_from_dttm = p.valid_from_dttm and p.party_rk = d.party_rk
                    where party_category_cd = "SCENARIO"
                    order by party_rk
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=scenariosd)

        /* pivot the table */
        proc transpose data=scenariosd out=scrdt;
             id UDF_NM;
             var UDF_VALUE;
             by PARTY_RK;
        run;

        /* drop the columns that are not part of a scenario. Drop header_id and routing_group_id that will be
         * populated based on the relationships */
        data scrd;
            set scrdt;
            drop _LABEL_  _NAME_ ;
        run;

        /* select all NUMERIC UDF columns for all scenarios */
        proc sql;
            create table scenariosn as
            select  p.party_rk, n.udf_nm, n.udf_value
                    from ecm_db.party_live p
                    inner join ecm_db.party_udf_num_value n
                    on n.udf_table_nm = "PARTY"
                       and n.valid_from_dttm = p.valid_from_dttm and p.party_rk = n.party_rk
                    where party_category_cd = "SCENARIO"
                    order by party_rk
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=scenariosn)

        /* pivot the table */
        proc transpose data=scenariosn out=scrnt;
             id  UDF_NM;
             var UDF_VALUE;
             by  PARTY_RK;
        run;

        data scrn;
            set scrnt;
            drop _LABEL_  _NAME_;
        run;

        /* join all UDF columns in one table */
        proc sql;
            create table SCRALL0
                as select c.*, d.*, n.*, p.VERSION_NO
                from scrc c
                left join scrd d
                    on c.party_rk = d.party_rk
                left join scrn n
                    on c.party_rk = n.party_rk
                inner join ecm_db.party_live p
                  on c.party_rk = p.party_rk
                ;
        quit;
        %fcf_sa_error_if_empty(tab_obs=&SQLOBS, err_msg=sa_no_scenarios);

        proc sql;
            proc sql;
                create table scrnondel1
                as select *
                from scrall0
            where logical_delete_ind ne "Y"
            ;
        quit;
        proc sql;
            create table scrdel
                as select c.*, d.*, n.*
                from scrc c
                left join scrd d
                    on c.party_rk = d.party_rk
                left join scrn n
                    on c.party_rk = n.party_rk
            where logical_delete_ind eq "Y"
                  and exists (select scenario_name from  seg_kc.fsk_scenario s1
                                  where c.scenario_name = s1.scenario_name
                                        and s1.logical_delete_ind = "N"
                                        and s1.CURRENT_IND = "Y"
                                        and s1.scenario_type_code ne 'VSD'
                                        and s1.scenario_id > 0
                             )
            ;
        quit;


        /* ECM creates relationship between header and scenario, add header_id based on the relationship */
        proc sql;
            create table hdrXscr as
            select * from ecm_db.party_x_party
            where member_type_cd = "hdr_X_scr";
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=hdrXscr)
        /* ECM creates relationship between scenario and routing group, add routing_group_id based on the relationship */
        proc sql;
            create table scrXrg as
            select * from ecm_db.party_x_party
            where member_type_cd = "scr_X_rg";
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=scrXrg)

        /* exclude scenarios from deleted headers or scenarios not included in the headers  */
        %if &rg_empty eq 1 %then %do;
            proc sql;
                create table scrnondel
                as select
                       p.valid_from_dttm as VALID_FROM_DTTM,
                       p.CREATE_USER_ID,
                       s.*,
                       h.header_id,
                       . as routing_group_id
                   from scrnondel1 s
                   left join hdrXscr x
                       on x.member_party_rk = s.party_rk
                   inner join hdrall h
                       on x.party_rk = h.party_rk
                   left join ecm_db.party_live p
                       on s.party_rk = p.party_rk
                ;
            quit;
        %end;
        %else %do;
            proc sql;
                create table scrnondel
                as select
                       p.valid_from_dttm as VALID_FROM_DTTM,
                       p.CREATE_USER_ID,
                       s.*,
                       h.header_id,
                       r.routing_group_id
                   from scrnondel1 s
                   left join hdrXscr x
                      on x.member_party_rk = s.party_rk
                   left join scrXrg xr
                      on xr.party_rk = s.party_rk
                   inner join hdrall h
                      on x.party_rk = h.party_rk
                   left join rgall r
                      on xr.member_party_rk = r.party_rk
                   left join ecm_db.party_live p
                      on s.party_rk = p.party_rk
                ;
            quit;
        %end;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=scrnondel)

        /* fix the datatypes to match FCF tables*/
        data scrfix1;
            length SCENARIO_CODE_LOCATION_DESC $255; /* need this in case all scenarios do not have source */
            length LSTUPDATE_DATE 8;
            format LSTUPDATE_DATE datetime20.;
            set scrnondel (rename=(
                            SCENARIO_ID                  =X_SCENARIO_ID
                            HEADER_ID                    =X_HEADER_ID
                            ROUTING_GROUP_ID             =X_ROUTING_GROUP_ID
                        ));
            drop _LABEL_  _NAME_ x_header_id X_SCENARIO_ID X_HEADER_ID X_ROUTING_GROUP_ID X_CREATE_USER_ID VERSION_NO;
            length DAV_FILE_NAME $255;


            if LOGICAL_DELETE_IND eq "Y" then delete; /* not not deploy deleted scenarios */
            if X_SCENARIO_ID = ' ' then delete;       /* skip corrupted data - all records must have scenario_id */
            LSTUPDATE_DATE = input(put (VALID_FROM_DTTM, datetime20.), datetime20.);
            drop VALID_FROM_DTTM;
            ECM_SCENARIO_ID              =input(X_SCENARIO_ID, BEST32. );
            HEADER_ID                    =input(X_HEADER_ID, BEST32. );
            ROUTING_GROUP_ID             =input(X_ROUTING_GROUP_ID, BEST32. );
            if LOGICAL_DELETE_IND ne "Y"
               AND CURRENT_IND ne "N"
               and (SCENARIO_TYPE_CODE eq "CUS" or SCENARIO_TYPE_CODE eq "HPS")
            then do;
                DAV_FILE_NAME = SCENARIO_CODE_LOCATION_DESC;
                if SCENARIO_CODE_LOCATION_DESC ne " " then SCENARIO_CODE_LOCATION_DESC  ="&fcf_sa_scr_path" || left(trim(SCENARIO_NAME)) ||  '_v_' || trim(left(VERSION_NO)) || ".sas";
            end;
            else do;
                DAV_FILE_NAME = " ";
                SCENARIO_CODE_LOCATION_DESC = " ";
            end;
            CURRENT_IND = "Y";
            /* create_date cannot be NULL in FCF */
            if CREATE_DATE = . then CREATE_DATE = DATETIME();
            if VERSION_NUMBER = . then VERSION_NUMBER = 1;
            output;
        run;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=scrfix)

        /* find scenarios from deleted headers or scenarios not included in the headers
         * they must be marked as deleted in KC if they exists there (non-deleted and current)
         * scrfix1 has only non-deleted scenarios included in a non-deleted header (ECM)
         * scrnondel1 has all non-deleted scenarios in ECM
         * also scenarios that were renamed have to be deleted too
        */
        proc sql;
            create table scrdel1 as
                select s.SCENARIO_NAME
                from  scrnondel1 s
                where not exists (select scenario_name from  scrfix1 s1 where s.scenario_name = s1.scenario_name)
                      and exists (select scenario_name from  seg_kc.fsk_scenario s1
                                  where s.scenario_name = s1.scenario_name
                                        and s1.logical_delete_ind = "N"
                                        and s1.CURRENT_IND = "Y"
                                        and s1.scenario_type_code ne 'VSD'
                                        and s1.scenario_id > 0
                                 )
                UNION
                select s1.SCENARIO_NAME
                from  seg_kc.fsk_scenario s1
                where not exists (select scenario_name from  scrfix1 s where s.scenario_name = s1.scenario_name)
                    and s1.logical_delete_ind = "N"
                    and s1.CURRENT_IND = "Y"
                    and s1.scenario_type_code ne 'VSD'
                    and s1.scenario_id > 0
            ;
        quit;

        /* do not update scenarios that have not changed
         * but update those where header have changed from DELETED to UNDELETED
         */
        proc sql;
            delete from scrfix1 s
            where exists (select * from seg_kc.fsk_scenario f where f.SCENARIO_NAME eq s.SCENARIO_NAME
                                                                    and (f.LSTUPDATE_DATE eq s.LSTUPDATE_DATE)
                                                                    and f.CURRENT_IND = "Y"
                                                                    and f.logical_delete_ind = "N"
                                                                    and f.scenario_type_code ne 'VSD'
                                                                    and f.routing_group_id = s.routing_group_id
                         )
                  and not exists (select * from HDRUNDELETED h where s.header_id = h.header_id)
            ;
        quit;

        %let table_size = 0;
        proc sql;
            select count (*) into :table_size from scrfix1;
        quit;
        %put &table_size;
        %let max_scen_id = 0;
        proc sql noprint;
            select max(scenario_id) into :max_scen_id from seg_kc.fsk_scenario;
        quit;
        %let next_id=0;
        %fcf_inc_fsk_sequence (next_id_var=next_id, inc_size=&table_size, max_used_id=&max_scen_id);
        %put &=next_id;

        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_sequence)

        data scrfix;
            set scrfix1;
            length SCENARIO_ID 8.;
            retain SCENARIO_ID;
            if _N_ = 1 then do;
                SCENARIO_ID = &next_id;
            end;
            else do;
                SCENARIO_ID + 1;
            end;
        run;

        /* make the dataset of the same type as fsk_scenario so that insert will have the same order of columns */
        data scrupd;
            set seg_kc.fsk_scenario;
            stop;
        run;

        proc append base=scrupd data=scrfix FORCE NOWARN;
        run;
    %mend fcf_sa_extract_scenarios;

    %fcf_sa_extract_scenarios()

    /****************************************************************************
     ****************************************************************************
     **          Create data for fsk_scenario_parameter table
     ****************************************************************************
     ****************************************************************************/
    %let parm_cnt=0;
    %macro fcf_sa_extract_scenario_parms();
        /* process fsk_scenario_parameter */
        /* only CHAR UDF columns are used for X_SCR_P */
        proc sql;
            create table paramsc as
            select  p.party_rk, c.udf_nm, c.udf_value length=&fcf_max_udf_len as UDF_VALUE, c.row_no
                    from ecm_db.party_live p
                    inner join ecm_db.party_udf_char_value c
                    on c.udf_table_nm = "X_SCR_P"
                       and c.valid_from_dttm = p.valid_from_dttm and p.party_rk = c.party_rk
                    where p.party_category_cd = "SCENARIO"
                    order by party_rk, row_no
            ;
        quit;
        %let parm_cnt=&SQLOBS;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=paramsc)

        /* make the dataset of the same type as fsk_scenario_parameter so that insert will have the same order of params */
        data prmins;
            set seg_kc.fsk_scenario_parameter;
            stop;
        run;

        %if &parm_cnt ne 0 %then %do;
            proc transpose data=paramsc out=prmall;
                 id UDF_NM;
                 var UDF_VALUE;
                 by PARTY_RK ROW_NO;
            run;

            /* fix the datatypes to match FCF tables*/
            data prmfix;
                set prmall (rename=(scenario_id=x_scenario_id));
                drop _LABEL_ ROW_NO _NAME_  x_scenario_id;
                length ECM_SCENARIO_ID 8.;
                if x_scenario_id = ' ' then delete; /* skip corrupted data - all records must have scenario_id */
                ECM_SCENARIO_ID = input(x_scenario_id, BEST32. );
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=prmfix)

            /* change the scenario ref to the new ids and remove parameters for deleted scenarios */
            proc sql;
                create table prmfull
                as select f.*, s.scenario_id
                   from prmfix f
                   inner join scrfix s
                   on f.ECM_SCENARIO_ID = s.ECM_SCENARIO_ID and s.party_rk = f.party_rk
                ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=prmfix)

            proc append base=prmins data=prmfull FORCE NOWARN;
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=prmfix)
        %end;
    %mend fcf_sa_extract_scenario_parms;

    %fcf_sa_extract_scenario_parms();

    /****************************************************************************
     ****************************************************************************
     **          Create data for fsk_scenario_alert_column table
     ****************************************************************************
     ****************************************************************************/
    /* only CHAR UDF columns are used for X_AC */
    %macro fcf_sa_process_alert_columns();
        /* make the dataset of the same type as fsk_scenario_alert_column so that insert will have the same order of alertcols */
        data acins;
            set seg_kc.fsk_scenario_alert_column;
            stop;
        run;

        proc sql;
            create table alertcolsc as
            select  p.party_rk, c.udf_nm, c.udf_value  length=&fcf_max_udf_len as UDF_VALUE, c.row_no
                    from ecm_db.party_live p
                         inner join ecm_db.party_udf_char_value c
                                    on c.udf_table_nm = "X_AC"
                                       and c.valid_from_dttm = p.valid_from_dttm
                                       and p.party_rk = c.party_rk
                    where p.party_category_cd = "SCENARIO"
                    order by party_rk, row_no
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=alertcolsc)

        %if &SQLOBS ne 0 %then %do;

            proc transpose data=alertcolsc out=acall;
                 id UDF_NM;
                 var UDF_VALUE;
                 by PARTY_RK ROW_NO;
            run;


            /* TODO: check that SCENARIO_ID column exists. Fix other code like this */
            /* fix the datatypes to match FCF tables */
            data acfix;
                set acall (rename=(scenario_id=x_scenario_id));
                drop _LABEL_ ROW_NO _NAME_ PARTY_RK x_scenario_id;
                length ECM_SCENARIO_ID 8.;
                if x_scenario_id = ' ' then delete; /* skip corrupted data - all records must have scenario_id */
                ECM_SCENARIO_ID = input(x_scenario_id, BEST32. );
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=acfix)

            /* change the scenario ref to the new ids and exclude columns for deleted scenarios */
            proc sql;
                create table acfull
                as select f.*, s.scenario_id
                   from acfix f
                        inner join scrfix s
                              on f.ECM_SCENARIO_ID = s.ECM_SCENARIO_ID
                ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=acfix)

            proc append base=acins data=acfull FORCE NOWARN;
            run;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=acfix)

            %fcf_debug_copy();
            proc datasets lib=work nolist;
               delete acfix acfull acall;
            run;
         %end;
    %mend fcf_sa_process_alert_columns;

    %fcf_sa_process_alert_columns()

    /***************************************************************************/

    %macro fcf_sa_update_tables();
        /****************************************************************************
         ****************************************************************************
         **          UPDATE fsk_header table
         ****************************************************************************
         ****************************************************************************/
        /* headers that are marked as deleted in ECM must be marked as deleted in KC
         * also headers that no longer exist in ECM must be marked as deleted in KC
         * Both conditions can be checked by verifying if header_id exists in hdrnondel
         */

        proc sql;
            create table hdr_del_list as
            select * from seg_kc.fsk_header h
            where header_id > 0
                  and LOGICAL_DELETE_IND = "N"
                  and not exists ( select header_id from hdrnondel f where h.header_id = f.header_id )
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=hdr_del_list)

        proc sql;
            update seg_kc.fsk_header h
                set LOGICAL_DELETE_IND = "Y",
                    LSTUPDATE_DATE = datetime()
            where header_id > 0
                  and LOGICAL_DELETE_IND = "N"
                  and not exists ( select header_id from hdrnondel f where h.header_id = f.header_id )
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_header)

        /* update existing records */
        data seg_kc.fsk_header;
             modify seg_kc.fsk_header
                    HDRUPD
                    UPDATEMODE=NOMISSINGCHECK;
             BY header_id;
             if _iorc_= 0 then replace;
        run;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_header)

        /* add new headers */
        proc sql;
            insert into seg_kc.fsk_header
                select * from hdrins
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_header)

        /***************** update FSK_HEADER_ELEMENT ***********************************/
        proc sql;
            delete from seg_kc.fsk_header_element;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_header_element)

        %if &elem_cnt ne 0 %then %do;
            proc sql;
                insert into seg_kc.fsk_header_element
                    select * from colins
                ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_header_element)
        %end;
        /****************************************************************************
         ****************************************************************************
         **          add new records to the FSK_SCENARIO table
         ****************************************************************************
         ****************************************************************************/
        /* We have to keep all existing fsk_scenario and fsk_scenario_parameter records
         * for audit purposes. So we just mark all existing scenarios that were changed as non-current,
         * add new records assigning new scenario_id values and add new records to
         * fsk_scenario_parameter modifying references using the new scenario_id values */
        /* mark all existing updated records as non current */
        proc sql;
            update seg_kc.fsk_scenario f
                set CURRENT_IND = "N"
            where exists (select * from scrupd s where f.scenario_name eq s.scenario_name and f.scenario_type_code ne 'VSD')
                  and scenario_id ne -1
                  and CURRENT_IND="Y"
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_scenario)

        /* mark all deleted records as deleted */
        proc sql;
            update seg_kc.fsk_scenario f
                set LOGICAL_DELETE_IND = "Y"
            where ( exists (select * from scrdel d where f.scenario_name eq d.scenario_name)
                    or
                    exists (select * from scrdel1 d where f.scenario_name eq d.scenario_name)
                  )
                  and scenario_id ne -1
                  and f.scenario_type_code ne 'VSD'
                  and f.LOGICAL_DELETE_IND = "N"
                  and f.CURRENT_IND="Y"
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_scenario)

        /* find scenarios that are not included in active (non-deleted) headers */
        proc sql;
            create table scrdel2 as
                select * from  seg_kc.fsk_scenario s
                where not exists (select * from seg_kc.fsk_header h
                                    where s.header_id = h.header_id
                                          and h.LOGICAL_DELETE_IND = "N")
                      and s.LOGICAL_DELETE_IND = "N" and s.CURRENT_IND="Y"
                      and s.scenario_id ne -1
                      and s.scenario_type_code ne 'VSD'
            ;
        quit;
        %if &SQLOBS ne 0 %then %do;
            proc sql;
                update seg_kc.fsk_scenario s
                    set LOGICAL_DELETE_IND = "Y"
                where not exists (select * from seg_kc.fsk_header h
                                            where s.header_id = h.header_id
                                                  and h.LOGICAL_DELETE_IND = "N")
                      and s.LOGICAL_DELETE_IND = "N" and s.CURRENT_IND="Y"
                      and s.scenario_id ne -1
                      and s.scenario_type_code ne 'VSD'
                ;
            quit;
        %end;

        proc sql;
            insert into seg_kc.fsk_scenario
                select * from scrupd
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_scenario)

        /****************************************************************************
         ****************************************************************************
         **          UPDATE FSK_SCENARIO_PARAMETER table
         ****************************************************************************
         ****************************************************************************/
        /* do not delete old records - they are used in FCF when displaying alert */
        %if &parm_cnt ne 0 %then %do;
            proc sql;
                insert into seg_kc.fsk_scenario_parameter
                    select * from prmins
                ;
            quit;
            %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_scenario_parameter)
        %end;
        /****************************************************************************
         ****************************************************************************
         **          UPDATE FSK_SCENARIO_ALERT_COLUMN table
         ****************************************************************************
         ****************************************************************************/
        /* do not delete old records - they are used in FCF when displaying alert
         *   proc sql;
         *       delete from seg_kc.fsk_scenario_alert_column
         *       ;
         *   quit;
         *   %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_scenario_parameter)
         */
        proc sql;
            insert into seg_kc.fsk_scenario_alert_column
                select * from acins
            ;
        quit;
        %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fsk_scenario_parameter)
    %mend fcf_sa_update_tables;
    %fcf_sa_update_tables();
    %fcf_clear_sync_lock(fcf_sync_job_ds=&fcf_sync_job_ds);

    /****************************************************************************
     ****************************************************************************
     **          Download scenario code
     ****************************************************************************
     ****************************************************************************/
    %fcf_sa_download_scenarios(dav_server_root=&fcf_dav_root, dav_server_path=&scr_dav_path, filenames_ds=scrfix)

    /****************************************************************************
     ****************************************************************************
     **          Download header code
     ****************************************************************************
     ****************************************************************************/
    %fcf_sa_download_headers(dav_server_root=&fcf_dav_root, dav_server_path=&hdr_dav_path, filenames_ds=hdrnondel)

    %fcf_debug_copy();

    %macro fcf_enable_formats;
            %if &enable_formats eq Y %then %do;
                %fcf_create_formats;
            %end;
            %else %do;
                %put NOTE: FCF: Formats are not enabled.;
            %end;
    %mend fcf_enable_formats;
    %fcf_enable_formats;
    %fcf_sa_abort_on_error(msg=sa_processing_error, p1=fmts.formats)

    /******************* delete temp datasets ***********************************/
    %fcf_debug_copy();
    proc datasets lib=work nolist;
       delete  hdrdel hdrall rgall elements hdrxscr scrfix1 scrxrg  dataset_list
       hdrins hdrc hdrd hdrdt hdrjoin hdrn hdrnt headersc headersd headersn HDRUNDELETED
       paramsc prmall prmfix prmfull alertcolsc scenariosc scenariosd scenariosn
       scrnondel1 scrnondel scrall0 scrct scrc scrd scrdt scrn scrnt
       colall colc colfull coln colnt columnsc columnsn acins colins prmins scrfix ;
    quit;

%mend fcf_sa_deploy_scenarios;
